@@ -1,4 +1,4 @@
-/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -17,6 +17,8 @@
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <sys/types.h>   // OpenBSD
+#include <netinet/tcp.h> // TCP_FASTOPEN
 #include <netinet/in.h>
 #include <poll.h>
 #include <stdbool.h>
@@ -30,16 +32,6 @@
 #include "contrib/sockaddr.h"
 #include "contrib/time.h"
 
-/*
- * OS X doesn't support MSG_NOSIGNAL. Use SO_NOSIGPIPE socket option instead.
- */
-#if defined(__APPLE__) && !defined(MSG_NOSIGNAL)
-#  define MSG_NOSIGNAL 0
-#  define osx_block_sigpipe(sock) sockopt_enable(sock, SOL_SOCKET, SO_NOSIGPIPE)
-#else
-#  define osx_block_sigpipe(sock) KNOT_EOK
-#endif
-
 /*!
  * \brief Enable socket option.
  */
@@ -77,10 +69,16 @@ static int socket_create(int family, int type, int proto)
 	}
 #endif
 
-	int ret = osx_block_sigpipe(sock);
+/*
+ * OS X doesn't support MSG_NOSIGNAL. Use SO_NOSIGPIPE socket option instead.
+ */
+#if defined(__APPLE__) && !defined(MSG_NOSIGNAL)
+#define MSG_NOSIGNAL 0
+	int ret = sockopt_enable(sock, SOL_SOCKET, SO_NOSIGPIPE);
 	if (ret != KNOT_EOK) {
 		return ret;
 	}
+#endif
 
 	return sock;
 }
@@ -164,7 +162,7 @@ static void unlink_unix_socket(const struct sockaddr_storage *addr)
 	unlink(path);
 }
 
-int net_bound_socket(int type, const struct sockaddr_storage *addr, enum net_flags flags)
+int net_bound_socket(int type, const struct sockaddr_storage *addr, net_bind_flag_t flags)
 {
 	/* Create socket. */
 	int sock = net_unbound_socket(type, addr);
@@ -184,6 +182,18 @@ int net_bound_socket(int type, const struct sockaddr_storage *addr, enum net_fla
 		return ret;
 	}
 
+#if defined(__linux__)
+	/* Set MSS (Maximum Segment Size) limit. */
+	if (addr->ss_family != AF_UNIX && type == SOCK_STREAM) {
+		const int mss = KNOT_TCP_MSS;
+		if (setsockopt(sock, IPPROTO_TCP, TCP_MAXSEG, &mss, sizeof(mss)) != 0) {
+			ret = knot_map_errno();
+			close(sock);
+			return ret;
+		}
+	}
+#endif
+
 	/* Don't bind IPv4 for IPv6 any address. */
 	if (addr->ss_family == AF_INET6) {
 		ret = sockopt_enable(sock, IPPROTO_IPV6, IPV6_V6ONLY);
@@ -222,8 +232,30 @@ int net_bound_socket(int type, const struct sockaddr_storage *addr, enum net_fla
 	return sock;
 }
 
+static int tfo_connect(int sock, const struct sockaddr_storage *addr)
+{
+#if defined(__linux__)
+	/* connect() will be called implicitly with sendmsg(). */
+	return KNOT_EOK;
+#elif defined(__FreeBSD__)
+	return sockopt_enable(sock, IPPROTO_TCP, TCP_FASTOPEN);
+#elif defined(__APPLE__)
+	/* Connection is performed lazily when first data is sent. */
+	sa_endpoints_t ep = {
+		.sae_dstaddr = (const struct sockaddr *)addr,
+		.sae_dstaddrlen = sockaddr_len(addr)
+	};
+	int flags =  CONNECT_DATA_IDEMPOTENT | CONNECT_RESUME_ON_READ_WRITE;
+
+	int ret = connectx(sock, &ep, SAE_ASSOCID_ANY, flags, NULL, 0, NULL, NULL);
+	return (ret == 0 ? KNOT_EOK : knot_map_errno());
+#else
+	return KNOT_ENOTSUP;
+#endif
+}
+
 int net_connected_socket(int type, const struct sockaddr_storage *dst_addr,
-                         const struct sockaddr_storage *src_addr)
+                         const struct sockaddr_storage *src_addr, bool tfo)
 {
 	if (dst_addr == NULL) {
 		return KNOT_EINVAL;
@@ -246,16 +278,42 @@ int net_connected_socket(int type, const struct sockaddr_storage *dst_addr,
 	}
 
 	/* Connect to destination. */
-	int ret = connect(sock, (const struct sockaddr *)dst_addr, sockaddr_len(dst_addr));
-	if (ret != 0 && errno != EINPROGRESS) {
-		ret = knot_map_errno();
-		close(sock);
-		return ret;
+	if (tfo && net_is_stream(sock)) {
+		int ret = tfo_connect(sock, dst_addr);
+		if (ret != KNOT_EOK) {
+			close(sock);
+			return ret;
+		}
+	} else {
+		int ret = connect(sock, (const struct sockaddr *)dst_addr,
+		                  sockaddr_len(dst_addr));
+		if (ret != 0 && errno != EINPROGRESS) {
+			ret = knot_map_errno();
+			close(sock);
+			return ret;
+		}
 	}
 
 	return sock;
 }
 
+int net_bound_tfo(int sock, int backlog)
+{
+#if defined(TCP_FASTOPEN)
+#if defined(__APPLE__)
+	if (backlog > 0) {
+		backlog = 1; // just on-off switch on macOS
+	}
+#endif
+	if (setsockopt(sock, IPPROTO_TCP, TCP_FASTOPEN, &backlog, sizeof(backlog)) != 0) {
+		return knot_map_errno();
+	}
+
+	return KNOT_EOK;
+#endif
+	return KNOT_ENOTSUP;
+}
+
 bool net_is_connected(int sock)
 {
 	struct sockaddr_storage addr;
@@ -359,7 +417,7 @@ static bool wait_should_retry(int error)
  * \brief I/O operation callbacks.
  */
 struct io {
-	ssize_t (*process)(int sockfd, struct msghdr *msg);
+	ssize_t (*process)(int sockfd, struct msghdr *msg, int timeout_ms);
 	int (*wait)(int sockfd, int timeout_ms);
 };
 
@@ -426,7 +484,7 @@ static ssize_t io_exec(const struct io *io, int fd, struct msghdr *msg,
 
 	for (;;) {
 		/* Perform I/O. */
-		ssize_t ret = io->process(fd, msg);
+		ssize_t ret = io->process(fd, msg, *timeout_ptr);
 		if (ret == -1 && errno == EINTR) {
 			continue;
 		}
@@ -475,7 +533,7 @@ static ssize_t io_exec(const struct io *io, int fd, struct msghdr *msg,
 	return done;
 }
 
-static ssize_t recv_process(int fd, struct msghdr *msg)
+static ssize_t recv_process(int fd, struct msghdr *msg, int timeout_ms)
 {
 	return recvmsg(fd, msg, MSG_DONTWAIT | MSG_NOSIGNAL);
 }
@@ -495,7 +553,24 @@ static ssize_t recv_data(int sock, struct msghdr *msg, bool oneshot, int *timeou
 	return io_exec(&RECV_IO, sock, msg, oneshot, timeout_ptr);
 }
 
-static ssize_t send_process(int fd, struct msghdr *msg)
+static ssize_t send_process_tfo(int fd, struct msghdr *msg, int timeout_ms)
+{
+#if defined(__linux__)
+	int ret = sendmsg(fd, msg, MSG_FASTOPEN);
+	if (ret != 0 && errno == EINPROGRESS) {
+		if (poll_one(fd, POLLOUT, timeout_ms) != 1) {
+			errno = ETIMEDOUT;
+			return -1;
+		}
+		ret = sendmsg(fd, msg, MSG_NOSIGNAL);
+	}
+	return ret;
+#else
+	return sendmsg(fd, msg, MSG_NOSIGNAL);
+#endif
+}
+
+static ssize_t send_process(int fd, struct msghdr *msg, int timeout_ms)
 {
 	return sendmsg(fd, msg, MSG_NOSIGNAL);
 }
@@ -505,14 +580,18 @@ static int send_wait(int fd, int timeout_ms)
 	return poll_one(fd, POLLOUT, timeout_ms);
 }
 
-static ssize_t send_data(int sock, struct msghdr *msg, int *timeout_ptr)
+static ssize_t send_data(int sock, struct msghdr *msg, int *timeout_ptr, bool tfo)
 {
 	static const struct io SEND_IO = {
 		.process = send_process,
 		.wait = send_wait
 	};
+	static const struct io SEND_IO_TFO = {
+		.process = send_process_tfo,
+		.wait = send_wait
+	};
 
-	return io_exec(&SEND_IO, sock, msg, false, timeout_ptr);
+	return io_exec(tfo ? &SEND_IO_TFO : &SEND_IO, sock, msg, false, timeout_ptr);
 }
 
 /* -- generic stream and datagram I/O -------------------------------------- */
@@ -534,7 +613,7 @@ ssize_t net_base_send(int sock, const uint8_t *buffer, size_t size,
 	msg.msg_iov = &iov;
 	msg.msg_iovlen = 1;
 
-	int ret = send_data(sock, &msg, &timeout_ms);
+	int ret = send_data(sock, &msg, &timeout_ms, false);
 	if (ret < 0) {
 		return ret;
 	} else if (ret != size) {
@@ -587,7 +666,8 @@ ssize_t net_stream_recv(int sock, uint8_t *buffer, size_t size, int timeout_ms)
 
 /* -- DNS specific I/O ----------------------------------------------------- */
 
-ssize_t net_dns_tcp_send(int sock, const uint8_t *buffer, size_t size, int timeout_ms)
+ssize_t net_dns_tcp_send(int sock, const uint8_t *buffer, size_t size, int timeout_ms,
+                         struct sockaddr_storage *tfo_addr)
 {
 	if (sock < 0 || buffer == NULL || size > UINT16_MAX) {
 		return KNOT_EINVAL;
@@ -603,8 +683,10 @@ ssize_t net_dns_tcp_send(int sock, const uint8_t *buffer, size_t size, int timeo
 	struct msghdr msg = { 0 };
 	msg.msg_iov = iov;
 	msg.msg_iovlen = 2;
+	msg.msg_name = (void *)tfo_addr;
+	msg.msg_namelen = tfo_addr ? sizeof(*tfo_addr) : 0;
 
-	ssize_t ret = send_data(sock, &msg, &timeout_ms);
+	ssize_t ret = send_data(sock, &msg, &timeout_ms, tfo_addr != NULL);
 	if (ret < 0) {
 		return ret;
 	}
@@ -618,16 +700,18 @@ ssize_t net_dns_tcp_recv(int sock, uint8_t *buffer, size_t size, int timeout_ms)
 		return KNOT_EINVAL;
 	}
 
-	struct iovec iov = { 0 };
-	struct msghdr msg = { 0 };
-	msg.msg_iov = &iov;
-	msg.msg_iovlen = 1;
+	uint16_t pktsize = 0;
+
+	struct iovec iov = {
+		.iov_base = &pktsize,
+		.iov_len = sizeof(pktsize)
+	};
+	struct msghdr msg = {
+		.msg_iov = &iov,
+		.msg_iovlen = 1
+	};
 
 	/* Receive size. */
-	uint16_t pktsize = 0;
-	iov.iov_base = &pktsize;
-	iov.iov_len = sizeof(pktsize);
-
 	int ret = recv_data(sock, &msg, false, &timeout_ms);
 	if (ret != sizeof(pktsize)) {
 		return ret;
@@ -640,8 +724,6 @@ ssize_t net_dns_tcp_recv(int sock, uint8_t *buffer, size_t size, int timeout_ms)
 	}
 
 	/* Receive payload. */
-	msg.msg_iov = &iov;
-	msg.msg_iovlen = 1;
 	iov.iov_base = buffer;
 	iov.iov_len = pktsize;
 
