@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -11,7 +11,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+    along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #pragma once
@@ -20,6 +20,10 @@
 #include <time.h>
 #include <inttypes.h>
 
+#ifdef __APPLE__
+ #define st_mtim st_mtimespec
+#endif
+
 /*!
  * \brief Specify output format for knot_time_print().
  */
@@ -117,6 +121,11 @@ inline static knot_time_t knot_time_add(knot_time_t since, knot_timediff_t howlo
 	return (since != 0 ? since + howlong : since);
 }
 
+inline static knot_time_t knot_time_plus(knot_time_t a, knot_time_t b)
+{
+	return ((a && b) ? a + b : 0);
+}
+
 /*!
  * \brief Convert uint32_t-encoded timestamp to knot_time_t.
  *
