@@ -20,6 +20,17 @@
 
 struct zone_contents;
 
+/*!
+ * \brief Validate if given zone is valid catalog.
+ *
+ * \param zone   Catalog zone in question.
+ *
+ * \retval KNOT_EZONEINVAL   Invalid varsion record.
+ * \retval KNOT_EISRECORD    Some of single-record RRSets has multiple RRs.
+ * \return KNOT_EOK          All OK.
+ */
+int catalog_zone_verify(const struct zone_contents *zone);
+
 /*!
  * \brief Iterate over PTR records in given zone contents and add members to catalog update.
  *
@@ -27,7 +38,6 @@ struct zone_contents;
  * \param zone               Zone contents to be searched for member PTR records.
  * \param complete_contents  Complete zone contents (zone might be from a changeset).
  * \param remove             Add removals of found member zones.
- * \param check_ver          Do check catalog zone version record first.
  * \param check              Optional: existing catalog database to be checked for existence
  *                           of such record (useful for removals).
  * \param upd_count          Output: number of resulting updates to catalog database.
@@ -36,5 +46,4 @@ struct zone_contents;
  */
 int catalog_update_from_zone(catalog_update_t *u, struct zone_contents *zone,
                              const struct zone_contents *complete_contents,
-                             bool remove, bool check_ver, catalog_t *check,
-                             ssize_t *upd_count);
+                             bool remove, catalog_t *check, ssize_t *upd_count);
