@@ -1,4 +1,4 @@
-/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2022 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -171,6 +171,15 @@ static const knot_lookup_t catalog_roles[] = {
 	{ 0, NULL }
 };
 
+static const knot_lookup_t dbus_events[] = {
+	{ DBUS_EVENT_NONE,            "none" },
+	{ DBUS_EVENT_RUNNING,         "running" },
+	{ DBUS_EVENT_ZONE_UPDATED,    "zone-updated" },
+	{ DBUS_EVENT_ZONE_SUBMISSION, "ksk-submission" },
+	{ DBUS_EVENT_ZONE_INVALID,    "dnssec-invalid" },
+	{ 0, NULL }
+};
+
 static const yp_item_t desc_module[] = {
 	{ C_ID,      YP_TSTR, YP_VNONE, YP_FNONE, { check_module_id } },
 	{ C_FILE,    YP_TSTR, YP_VNONE },
@@ -210,6 +219,7 @@ static const yp_item_t desc_server[] = {
 	                                                1232, YP_SSIZE } },
 	{ C_ECS,                  YP_TBOOL, YP_VNONE },
 	{ C_ANS_ROTATION,         YP_TBOOL, YP_VNONE },
+	{ C_DBUS_EVENT,           YP_TOPT,  YP_VOPT = { dbus_events, DBUS_EVENT_NONE }, YP_FMULTI },
 	{ C_LISTEN,               YP_TADDR, YP_VADDR = { 53 }, YP_FMULTI, { check_listen } },
 	{ C_COMMENT,              YP_TSTR,  YP_VNONE },
 	// Legacy items.
@@ -330,12 +340,11 @@ static const yp_item_t desc_acl[] = {
 
 static const yp_item_t desc_submission[] = {
 	{ C_ID,           YP_TSTR, YP_VNONE },
-	{ C_PARENT,       YP_TREF, YP_VREF = { C_RMT }, YP_FMULTI | CONF_IO_FRLD_ZONES,
-	                           { check_ref } },
-	{ C_CHK_INTERVAL, YP_TINT, YP_VINT = { 1, UINT32_MAX, HOURS(1), YP_STIME },
-	                           CONF_IO_FRLD_ZONES },
+	{ C_PARENT,       YP_TREF, YP_VREF = { C_RMT }, YP_FMULTI, { check_ref } },
+	{ C_CHK_INTERVAL, YP_TINT, YP_VINT = { 1, UINT32_MAX, HOURS(1), YP_STIME } },
 	{ C_TIMEOUT,      YP_TINT, YP_VINT = { 0, UINT32_MAX, 0, YP_STIME },
 	                           CONF_IO_FRLD_ZONES },
+	{ C_PARENT_DELAY, YP_TINT, YP_VINT = { 0, UINT32_MAX, 0, YP_STIME } },
 	{ C_COMMENT,      YP_TSTR, YP_VNONE },
 	{ NULL }
 };
@@ -383,8 +392,10 @@ static const yp_item_t desc_policy[] = {
 	                                   { check_ref } },
 	{ C_DS_PUSH,             YP_TREF,  YP_VREF = { C_RMT }, YP_FMULTI | CONF_IO_FRLD_ZONES,
 	                                   { check_ref } },
-	{ C_CDS_CDNSKEY,         YP_TOPT,  YP_VOPT = { cds_cdnskey, CDS_CDNSKEY_ROLLOVER } },
-	{ C_CDS_DIGESTTYPE,      YP_TOPT,  YP_VOPT = { cds_digesttype, DNSSEC_KEY_DIGEST_SHA256 } },
+	{ C_CDS_CDNSKEY,         YP_TOPT,  YP_VOPT = { cds_cdnskey, CDS_CDNSKEY_ROLLOVER },
+	                                   CONF_IO_FRLD_ZONES },
+	{ C_CDS_DIGESTTYPE,      YP_TOPT,  YP_VOPT = { cds_digesttype, DNSSEC_KEY_DIGEST_SHA256 },
+	                                   CONF_IO_FRLD_ZONES },
 	{ C_OFFLINE_KSK,         YP_TBOOL, YP_VNONE, CONF_IO_FRLD_ZONES },
 	{ C_UNSAFE_OPERATION,    YP_TOPT,  YP_VOPT = { unsafe_operation, UNSAFE_NONE }, YP_FMULTI },
 	{ C_COMMENT,             YP_TSTR,  YP_VNONE },
