@@ -1,4 +1,4 @@
-/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2022 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -44,6 +44,7 @@
 #define C_CONFIG		"\x06""config"
 #define C_CTL			"\x07""control"
 #define C_DB			"\x08""database"
+#define C_DBUS_EVENT		"\x0A""dbus-event"
 #define C_DDNS_MASTER		"\x0B""ddns-master"
 #define C_DENY			"\x04""deny"
 #define C_DNSKEY_TTL		"\x0A""dnskey-ttl"
@@ -88,6 +89,7 @@
 #define C_NSID			"\x04""nsid"
 #define C_OFFLINE_KSK		"\x0B""offline-ksk"
 #define C_PARENT		"\x06""parent"
+#define C_PARENT_DELAY		"\x0C""parent-delay"
 #define C_PIDFILE		"\x07""pidfile"
 #define C_POLICY		"\x06""policy"
 #define C_PROPAG_DELAY		"\x11""propagation-delay"
@@ -231,6 +233,14 @@ enum {
 	CATALOG_ROLE_MEMBER    = 3,
 };
 
+enum {
+	DBUS_EVENT_NONE            = 0,
+	DBUS_EVENT_RUNNING         = (1 << 0),
+	DBUS_EVENT_ZONE_UPDATED    = (1 << 1),
+	DBUS_EVENT_ZONE_SUBMISSION = (1 << 2),
+	DBUS_EVENT_ZONE_INVALID    = (1 << 3),
+};
+
 extern const knot_lookup_t acl_actions[];
 
 extern const yp_item_t conf_schema[];
