@@ -547,6 +547,8 @@ int check_policy(
 						   C_DNSKEY_TTL, args->id, args->id_len);
 	conf_val_t zone_max_ttl = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_POLICY,
 						     C_ZONE_MAX_TLL, args->id, args->id_len);
+	conf_val_t nsec3 = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_POLICY,
+	                                      C_NSEC3, args->id, args->id_len);
 	conf_val_t nsec3_iters = conf_rawid_get_txn(args->extra->conf, args->extra->txn, C_POLICY,
 	                                            C_NSEC3_ITER, args->id, args->id_len);
 
@@ -625,10 +627,16 @@ int check_policy(
 	}
 #endif
 
-	uint16_t iters = conf_int(&nsec3_iters);
-	if (iters > 20) {
-		CONF_LOG(LOG_NOTICE, "policy[%s].nsec3-iterations=%u is too high, "
-		                     "recommended value is 0-10", args->id, iters);
+	if (conf_bool(&nsec3)) {
+		uint16_t iters = conf_int(&nsec3_iters);
+		if (nsec3_iters.code != KNOT_EOK && iters != 0) {
+			CONF_LOG(LOG_WARNING, "policy[%s].nsec3-iterations defaults to %u, "
+			                      "since version 3.2 the default becomes 0", args->id, iters);
+		}
+		if (iters > 20) {
+			CONF_LOG(LOG_NOTICE, "policy[%s].nsec3-iterations=%u is too high, "
+			                     "the recommended value is 0", args->id, iters);
+		}
 	}
 
 	return KNOT_EOK;
