@@ -1,4 +1,4 @@
-/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2022 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -74,6 +74,8 @@ typedef enum {
 	CTL_ZONE_KSK_SBM,
 	CTL_ZONE_FREEZE,
 	CTL_ZONE_THAW,
+	CTL_ZONE_XFR_FREEZE,
+	CTL_ZONE_XFR_THAW,
 
 	CTL_ZONE_READ,
 	CTL_ZONE_BEGIN,
