@@ -1,4 +1,4 @@
-/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2022 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -129,11 +129,17 @@ static void policy_load(knot_kasp_policy_t *policy, conf_t *conf, conf_val_t *id
 			}
 			conf_val_next(&val);
 		}
+
+		val = conf_id_get(conf, C_SBM, C_PARENT_DELAY, &ksk_sbm);
+		policy->ksk_sbm_delay = conf_int(&val);
 	}
 
 	val = conf_id_get(conf, C_POLICY, C_SIGNING_THREADS, id);
 	policy->signing_threads = conf_int(&val);
 
+	val = conf_id_get(conf, C_POLICY, C_DS_PUSH, id);
+	policy->ds_push = conf_val_count(&val) > 0;
+
 	val = conf_id_get(conf, C_POLICY, C_OFFLINE_KSK, id);
 	policy->offline_ksk = conf_bool(&val);
 
@@ -207,6 +213,8 @@ int kdnssec_ctx_init(conf_t *conf, kdnssec_ctx_t *ctx, const knot_dname_t *zone_
 		goto init_error;
 	}
 
+	ctx->dbus_event = conf->cache.srv_dbus_event;
+
 	ctx->now = knot_time();
 
 	return KNOT_EOK;
