@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -134,8 +134,8 @@ static bool params_deserialize(const MDB_val *val, key_params_t *params)
 	if (val->mv_size < 2 * sizeof(uint64_t)) {
 		return false;
 	}
-	uint64_t *_lengths = (uint64_t *)val->mv_data;
-	uint64_t keylen = be64toh(_lengths[0]), future = be64toh(_lengths[1]);
+	uint64_t keylen = knot_wire_read_u64(val->mv_data);
+	uint64_t future = knot_wire_read_u64(val->mv_data + sizeof(keylen));
 	uint8_t flags;
 
 	if ((params->public_key.data = malloc(keylen)) == NULL) {
@@ -161,7 +161,7 @@ static bool params_deserialize(const MDB_val *val, key_params_t *params)
 				return false;
 			}
 			// 'revoked' timer is part of 'future' section since it was added later
-			params->timing.revoke = be64toh(*(uint64_t *)(val->mv_data + val->mv_size - future));
+			params->timing.revoke = knot_wire_read_u64(val->mv_data + val->mv_size - future);
 		}
 
 		if ((flags & 0x02) && (params->is_ksk || !params->is_csk)) {
@@ -572,7 +572,7 @@ void kasp_db_ensure_init(knot_lmdb_db_t *db, conf_t *conf)
 {
 	if (db->path == NULL) {
 		char *kasp_dir = conf_db(conf, C_KASP_DB);
-		conf_val_t kasp_size = conf_db_param(conf, C_KASP_DB_MAX_SIZE, C_MAX_KASP_DB_SIZE);
+		conf_val_t kasp_size = conf_db_param(conf, C_KASP_DB_MAX_SIZE);
 		knot_lmdb_init(db, kasp_dir, conf_int(&kasp_size), 0, "keys_db");
 		free(kasp_dir);
 		assert(db->path != NULL);
