@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -401,7 +401,18 @@ int nsec_check_new_connects(zone_tree_t *tree, nsec_chain_iterate_data_t *data)
 	return zone_tree_apply(tree, nsec_check_prev_next, data);
 }
 
-extern bool nsec3_is_empty(zone_node_t *node, bool opt_out); // from nsec3-chain.c
+static int check_subtree_optout(zone_node_t *node, void *ctx)
+{
+	bool *res = ctx;
+	if ((node->flags & NODE_FLAGS_NONAUTH) || !*res) {
+		return KNOT_EOK;
+	}
+	if (node_nsec3_get(node) != NULL &&
+	    node_rdataset(node_nsec3_get(node), KNOT_RRTYPE_NSEC3) != NULL) {
+		*res = false;
+	}
+	return KNOT_EOK;
+}
 
 static int check_nsec_bitmap(zone_node_t *node, void *ctx)
 {
@@ -410,12 +421,24 @@ static int check_nsec_bitmap(zone_node_t *node, void *ctx)
 	const zone_node_t *nsec_node = node;
 	bool shall_no_nsec = node_no_nsec(node);
 	if (data->nsec3_params != NULL) {
+		if ((node->flags & NODE_FLAGS_DELETED) ||
+		    node_rrtype_exists(node, KNOT_RRTYPE_NSEC3)) {
+			// this can happen when checking nodes from adjust_ptrs
+			return KNOT_EOK;
+		}
 		nsec_node = node_nsec3_get(node);
 		shall_no_nsec = (node->flags & NODE_FLAGS_DELETED) ||
 		                (node->flags & NODE_FLAGS_NONAUTH);
 	}
-	bool may_no_nsec = (data->nsec3_params != NULL && nsec3_is_empty(node, true));
+	bool may_no_nsec = (data->nsec3_params != NULL && !(node->flags & NODE_FLAGS_SUBTREE_AUTH));
 	knot_rdataset_t *nsec = node_rdataset(nsec_node, data->nsec_type);
+	if (may_no_nsec && nsec == NULL) {
+		int ret = zone_tree_sub_apply(data->update->new_cont->nodes, node->owner,
+		                              true, check_subtree_optout, &may_no_nsec);
+		if (ret != KNOT_EOK) {
+			return ret;
+		}
+	}
 	if ((nsec == NULL || nsec->count != 1) && !shall_no_nsec && !may_no_nsec) {
 		data->update->validation_hint.node = (nsec_node == NULL ? node->owner : nsec_node->owner);
 		data->update->validation_hint.rrtype = KNOT_RRTYPE_ANY;
