@@ -898,10 +898,10 @@ int knot_zone_sign_update_dnskeys(zone_update_t *update,
 
 	if (!knot_rrset_empty(&add_r.cds)) {
 		ret = changeset_add_addition(&ch, &add_r.cds, CHANGESET_CHECK);
-		if (node_rrtype_exists(ch.add->apex, KNOT_RRTYPE_CDS)) {
+		if (dnssec_ctx->policy->ds_push && node_rrtype_exists(ch.add->apex, KNOT_RRTYPE_CDS)) {
 			// there is indeed a change to CDS
-			update->zone->timers.next_ds_push = time(NULL);
-			zone_events_schedule_now(update->zone, ZONE_EVENT_DS_PUSH);
+			update->zone->timers.next_ds_push = time(NULL) + dnssec_ctx->policy->propagation_delay;
+			zone_events_schedule_at(update->zone, ZONE_EVENT_DS_PUSH, update->zone->timers.next_ds_push);
 		}
 		CHECK_RET;
 	}
