@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,14 +14,16 @@
     along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
-#include "knot/journal/knot_lmdb.h"
-
 #include <stdarg.h>
 #include <stdio.h> // snprintf
 #include <stdlib.h>
 #include <sys/stat.h>
 #include <unistd.h>
 
+#include "knot/journal/knot_lmdb.h"
+
+#include "knot/conf/conf.h"
+#include "contrib/files.h"
 #include "contrib/wire_ctx.h"
 #include "libknot/dname.h"
 #include "libknot/endian.h"
@@ -78,7 +80,7 @@ void knot_lmdb_init(knot_lmdb_db_t *db, const char *path, size_t mapsize, unsign
 	db->dbname = dbname;
 	pthread_mutex_init(&db->opening_mutex, NULL);
 	db->maxdbs = 2;
-	db->maxreaders = 126/* = contrib/lmdb/mdb.c DEFAULT_READERS */;
+	db->maxreaders = conf_lmdb_readers(conf());
 }
 
 static bool lmdb_stat(const char *lmdb_path, struct stat *st)
@@ -140,9 +142,9 @@ static int lmdb_open(knot_lmdb_db_t *db)
 		return ret;
 	}
 
-	ret = mkdir(db->path, LMDB_DIR_MODE);
-	if (ret < 0 && errno != EEXIST) {
-		return -errno;
+	ret = make_dir(db->path, LMDB_DIR_MODE, true);
+	if (ret != KNOT_EOK) {
+		return ret;
 	}
 
 	long page_size = sysconf(_SC_PAGESIZE);
