@@ -490,7 +490,7 @@ static knotd_in_state_t pre_routine(knotd_in_state_t state, knot_pkt_t *pkt,
 		pthread_rwlock_rdlock(&ctx->signing_mutex);
 		ret = knot_parent_ds_query(mod->dnssec, mod->keyset, 1000);
 		pthread_rwlock_unlock(&ctx->signing_mutex);
-		if (ret != KNOT_EOK && mod->dnssec->policy->ksk_sbm_check_interval > 0) {
+		if (ret != KNOT_EOK && ret != KNOT_NO_READY_KEY && mod->dnssec->policy->ksk_sbm_check_interval > 0) {
 			ctx->event_parent_ds_q = mod->dnssec->now + mod->dnssec->policy->ksk_sbm_check_interval;
 		} else {
 			ctx->event_parent_ds_q = 0;
