@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -44,34 +44,17 @@ static knotd_state_t queryacl_process(knotd_state_t state, knot_pkt_t *pkt,
 	}
 
 	if (ctx->allow_addr.count > 0) {
-		if (!knotd_conf_addr_range_match(&ctx->allow_addr, qdata->params->remote)) {
+		const struct sockaddr_storage *addr = knotd_qdata_remote_addr(qdata);
+		if (!knotd_conf_addr_range_match(&ctx->allow_addr, addr)) {
 			qdata->rcode = KNOT_RCODE_NOTAUTH;
 			return KNOTD_STATE_FAIL;
 		}
 	}
 
 	if (ctx->allow_iface.count > 0) {
-		struct sockaddr_storage iface;
-		socklen_t iface_len = sizeof(iface);
-		struct sockaddr_storage *iface_ptr;
-
-		if (qdata->params->xdp_msg != NULL) {
-#ifdef ENABLE_XDP
-			iface_ptr = (struct sockaddr_storage *)&qdata->params->xdp_msg->ip_to;
-#else
-			assert(0);
-			return KNOTD_STATE_FAIL;
-#endif
-		} else {
-			if (getsockname(qdata->params->socket, (struct sockaddr *)&iface,
-			                &iface_len) != 0) {
-				knotd_mod_log(mod, LOG_ERR, "failed to get interface address");
-				return KNOTD_STATE_FAIL;
-			}
-			iface_ptr = &iface;
-		}
-
-		if (!knotd_conf_addr_range_match(&ctx->allow_iface, iface_ptr)) {
+		struct sockaddr_storage buff;
+		const struct sockaddr_storage *addr = knotd_qdata_local_addr(qdata, &buff);
+		if (!knotd_conf_addr_range_match(&ctx->allow_iface, addr)) {
 			qdata->rcode = KNOT_RCODE_NOTAUTH;
 			return KNOTD_STATE_FAIL;
 		}
