@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -266,12 +266,6 @@ static int follow_cname(knot_pkt_t *pkt, uint16_t rrtype, knotd_qdata_t *qdata)
 	default:          return KNOTD_IN_STATE_ERROR;
 	}
 
-	/* Check if RR count increased. */
-	if (pkt->rrset_count <= rr_count_before) {
-		qdata->extra->node = NULL; /* Act as if the name leads to nowhere. */
-		return KNOTD_IN_STATE_HIT;
-	}
-
 	/* Synthesize CNAME if followed DNAME. */
 	if (rrtype == KNOT_RRTYPE_DNAME) {
 		if (dname_cname_cannot_synth(&cname_rr, qdata->name)) {
@@ -297,6 +291,12 @@ static int follow_cname(knot_pkt_t *pkt, uint16_t rrtype, knotd_qdata_t *qdata)
 		}
 	}
 
+	/* Check if RR count increased. */
+	if (pkt->rrset_count <= rr_count_before) {
+		qdata->extra->node = NULL; /* Act as if the name leads to nowhere. */
+		return KNOTD_IN_STATE_HIT;
+	}
+
 	/* If node is a wildcard, follow only if we didn't visit the same node
 	 * earlier, as that would mean a CNAME loop. */
 	if (knot_dname_is_wildcard(cname_node->owner)) {
@@ -431,6 +431,8 @@ static int solve_name(int state, knot_pkt_t *pkt, knotd_qdata_t *qdata)
 
 static int solve_answer(int state, knot_pkt_t *pkt, knotd_qdata_t *qdata, void *ctx)
 {
+	int old_state = state;
+
 	/* Do not solve if already solved, e.g. in a module. */
 	if (state == KNOTD_IN_STATE_HIT) {
 		return state;
@@ -439,6 +441,11 @@ static int solve_answer(int state, knot_pkt_t *pkt, knotd_qdata_t *qdata, void *
 	/* Get answer to QNAME. */
 	state = solve_name(state, pkt, qdata);
 
+	/* Promote NODATA from a module if nothing found in zone. */
+	if (state == KNOTD_IN_STATE_MISS && old_state == KNOTD_IN_STATE_NODATA) {
+		state = old_state;
+	}
+
 	/* Is authoritative answer unless referral.
 	 * Must check before we chase the CNAME chain. */
 	if (state != KNOTD_IN_STATE_DELEG) {
@@ -666,7 +673,7 @@ int internet_process_query(knot_pkt_t *pkt, knotd_qdata_t *qdata)
 		}
 	}
 
-	NS_NEED_ZONE_CONTENTS(qdata, KNOT_RCODE_SERVFAIL); /* Expired */
+	NS_NEED_ZONE_CONTENTS(qdata); /* Expired */
 
 	/* Get answer to QNAME. */
 	qdata->name = knot_pkt_qname(qdata->query);
