@@ -1,4 +1,4 @@
-/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #include "knot/nameserver/process_query.h"
 
 /*! \brief Don't follow CNAME/DNAME chain beyond this depth. */
-#define CNAME_CHAIN_MAX 20
+#define CNAME_CHAIN_MAX 5
 
 /*!
  * \brief Answer query from an IN class zone.
@@ -49,13 +49,17 @@ int internet_process_query(knot_pkt_t *pkt, knotd_qdata_t *qdata);
 #define NS_NEED_ZONE(qdata, error_rcode) \
 	if ((qdata)->extra->zone == NULL) { \
 		qdata->rcode = (error_rcode); \
+		if ((error_rcode) == KNOT_RCODE_REFUSED) { \
+			qdata->rcode_ede = KNOT_EDNS_EDE_NOTAUTH; \
+		} \
 		return KNOT_STATE_FAIL; \
 	}
 
 /*! \brief Require existing zone contents or return failure. */
-#define NS_NEED_ZONE_CONTENTS(qdata, error_rcode) \
+#define NS_NEED_ZONE_CONTENTS(qdata) \
 	if ((qdata)->extra->contents == NULL) { \
-		qdata->rcode = (error_rcode); \
+		qdata->rcode = KNOT_RCODE_SERVFAIL; \
+		qdata->rcode_ede = KNOT_EDNS_EDE_INV_DATA; \
 		return KNOT_STATE_FAIL; \
 	}
 
@@ -67,8 +71,9 @@ int internet_process_query(knot_pkt_t *pkt, knotd_qdata_t *qdata);
 	}
 
 /*! \brief Require the zone not to be frozen. */
-#define NS_NEED_NOT_FROZEN(qdata, error_rcode) \
+#define NS_NEED_NOT_FROZEN(qdata) \
 	if ((qdata)->extra->zone->events.ufrozen) { \
-		(qdata)->rcode = (error_rcode); \
+		(qdata)->rcode = KNOT_RCODE_REFUSED; \
+		(qdata)->rcode_ede = KNOT_EDNS_EDE_NOT_READY; \
 		return KNOT_STATE_FAIL; \
 	}
