@@ -1,4 +1,4 @@
-/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -11,11 +11,12 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+    along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #pragma once
 
+#include "knot/journal/journal_read.h"
 #include "knot/nameserver/process_query.h"
 #include "knot/nameserver/xfr.h"
 #include "libknot/packet/pkt.h"
@@ -36,18 +37,19 @@ struct ixfr_proc {
 	/* Processing state. */
 	struct xfr_proc proc;
 	enum ixfr_state state;
+	bool in_remove_section;
 
 	/* Changes to be sent. */
-	chgset_ctx_list_t changesets;
+	journal_read_t *journal_ctx;
 
-	/* Currenty processed changeset. */
+	/* Currenty processed RRSet. */
 	knot_rrset_t cur_rr;
-	uint32_t soa_from;
-	uint32_t soa_to;
 
 	/* Processing context. */
 	knotd_qdata_t *qdata;
 	knot_mm_t *mm;
+	uint32_t soa_from;
+	uint32_t soa_to;
 };
 
 /*!
