@@ -17,8 +17,10 @@
 #include <assert.h>
 
 #include "libknot/attribute.h"
+#include "knot/common/unreachable.h"
 #include "knot/query/requestor.h"
 #include "libknot/errcode.h"
+#include "contrib/conn_pool.h"
 #include "contrib/mempattern.h"
 #include "contrib/net.h"
 #include "contrib/sockaddr.h"
@@ -41,11 +43,30 @@ static int request_ensure_connected(knot_request_t *request)
 	}
 
 	int sock_type = use_tcp(request) ? SOCK_STREAM : SOCK_DGRAM;
+
+	if (sock_type == SOCK_STREAM) {
+		request->fd = conn_pool_get(global_conn_pool,
+		                            &request->source,
+		                            &request->remote);
+		if (request->fd >= 0) {
+			return KNOT_EOK;
+		}
+
+		if (knot_unreachable_is(global_unreachables, &request->remote,
+		                        &request->source)) {
+			return KNOT_EUNREACH;
+		}
+	}
+
 	request->fd = net_connected_socket(sock_type,
 	                                   &request->remote,
 	                                   &request->source,
 	                                   request->flags & KNOT_REQUEST_TFO);
 	if (request->fd < 0) {
+		if (request->fd == KNOT_ETIMEOUT) {
+			knot_unreachable_add(global_unreachables, &request->remote,
+			                     &request->source);
+		}
 		return request->fd;
 	}
 
@@ -71,10 +92,16 @@ static int request_send(knot_request_t *request, int timeout_ms)
 	if (use_tcp(request)) {
 		ret = net_dns_tcp_send(request->fd, wire, wire_len, timeout_ms,
 		                       tfo_addr);
+		if (ret == KNOT_ETIMEOUT) { // Includes establishing conn which times out.
+			knot_unreachable_add(global_unreachables, &request->remote,
+			                     &request->source);
+		}
 	} else {
 		ret = net_dgram_send(request->fd, wire, wire_len, NULL);
 	}
-	if (ret != wire_len) {
+	if (ret < 0) {
+		return ret;
+	} else if (ret != wire_len) {
 		return KNOT_ECONN;
 	}
 
@@ -156,6 +183,13 @@ void knot_request_free(knot_request_t *request, knot_mm_t *mm)
 		return;
 	}
 
+	if (request->fd >= 0 && use_tcp(request) &&
+	    (request->flags & KNOT_REQUEST_KEEP)) {
+		request->fd = conn_pool_put(global_conn_pool,
+		                            &request->source,
+		                            &request->remote,
+		                            request->fd);
+	}
 	if (request->fd >= 0) {
 		close(request->fd);
 	}
@@ -313,6 +347,8 @@ int knot_requestor_exec(knot_requestor_t *requestor, knot_request_t *request,
 	/* Expect complete request. */
 	if (requestor->layer.state != KNOT_STATE_DONE) {
 		ret = KNOT_EPROCESSING;
+	} else {
+		request->flags |= KNOT_REQUEST_KEEP;
 	}
 
 	/* Verify last TSIG */
