@@ -27,6 +27,7 @@
 #include "knot/common/log.h"
 #include "knot/common/stats.h"
 #include "knot/common/systemd.h"
+#include "knot/common/unreachable.h"
 #include "knot/conf/confio.h"
 #include "knot/conf/migration.h"
 #include "knot/conf/module.h"
@@ -38,6 +39,7 @@
 #include "knot/zone/timers.h"
 #include "knot/zone/zonedb-load.h"
 #include "knot/worker/pool.h"
+#include "contrib/conn_pool.h"
 #include "contrib/net.h"
 #include "contrib/openbsd/strlcat.h"
 #include "contrib/os.h"
@@ -696,6 +698,11 @@ void server_deinit(server_t *server)
 
 	/* Close journal database if open. */
 	knot_lmdb_deinit(&server->journaldb);
+
+	/* Close and deinit connection pool. */
+	conn_pool_deinit(global_conn_pool);
+	global_conn_pool = NULL;
+	knot_unreachables_deinit(&global_unreachables);
 }
 
 static int server_init_handler(server_t *server, int index, int thread_count,
@@ -928,6 +935,7 @@ static void warn_server_reconfigure(conf_t *conf, server_t *server)
 	static bool warn_listen = true;
 	static bool warn_xdp_tcp = true;
 	static bool warn_route_check = true;
+	static bool warn_rmt_pool_limit = true;
 
 	if (warn_tcp_reuseport && conf->cache.srv_tcp_reuseport != conf_get_bool(conf, C_SRV, C_TCP_REUSEPORT)) {
 		log_warning(msg, &C_TCP_REUSEPORT[1]);
@@ -968,6 +976,12 @@ static void warn_server_reconfigure(conf_t *conf, server_t *server)
 		log_warning(msg, &C_ROUTE_CHECK[1]);
 		warn_route_check = false;
 	}
+
+	if (warn_rmt_pool_limit && global_conn_pool != NULL &&
+	    global_conn_pool->capacity != conf_get_int(conf, C_SRV, C_RMT_POOL_LIMIT)) {
+		log_warning(msg, &C_RMT_POOL_LIMIT[1]);
+		warn_rmt_pool_limit = false;
+	}
 }
 
 int server_reload(server_t *server)
@@ -1134,6 +1148,33 @@ static int reconfigure_timer_db(conf_t *conf, server_t *server)
 	return ret;
 }
 
+static int reconfigure_remote_pool(conf_t *conf)
+{
+	conf_val_t val = conf_get(conf, C_SRV, C_RMT_POOL_LIMIT);
+	size_t limit = conf_int(&val);
+	val = conf_get(conf, C_SRV, C_RMT_POOL_TIMEOUT);
+	knot_timediff_t timeout = conf_int(&val);
+	if (global_conn_pool == NULL && limit > 0) {
+		conn_pool_t *new_pool = conn_pool_init(limit, timeout);
+		if (new_pool == NULL) {
+			return KNOT_ENOMEM;
+		}
+		global_conn_pool = new_pool;
+	} else {
+		(void)conn_pool_timeout(global_conn_pool, timeout);
+	}
+
+	val = conf_get(conf, C_SRV, C_RMT_RETRY_DELAY);
+	int delay_ms = conf_int(&val);
+	if (global_unreachables == NULL && delay_ms > 0) {
+		global_unreachables = knot_unreachables_init(delay_ms);
+	} else {
+		(void)knot_unreachables_ttl(global_unreachables, delay_ms);
+	}
+
+	return KNOT_EOK;
+}
+
 int server_reconfigure(conf_t *conf, server_t *server)
 {
 	if (conf == NULL || server == NULL) {
@@ -1190,6 +1231,12 @@ int server_reconfigure(conf_t *conf, server_t *server)
 		          knot_strerror(ret));
 	}
 
+	/* Reconfigure connection pool. */
+	if ((ret = reconfigure_remote_pool(conf)) != KNOT_EOK) {
+		log_error("failed to reconfigure remote pool (%s)",
+		          knot_strerror(ret));
+	}
+
 	return KNOT_EOK;
 }
 
