@@ -1,4 +1,4 @@
-/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2019 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -11,7 +11,7 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+    along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #pragma once
@@ -26,7 +26,6 @@
 typedef enum {
 	CHANGESET_NONE = 0,
 	CHANGESET_CHECK = 1 << 0, /*! Perform redundancy check on additions/removals */
-	CHANGESET_CHECK_CANCELOUT = 1 << 1, /*! Do the complete cancelout on addition/removal/merge (depends on CHANGESET_CHECK */
 } changeset_flag_t;
 
 /*! \brief One zone change, from 'soa_from' to 'soa_to'. */
@@ -43,6 +42,9 @@ typedef struct {
 /*! \brief Changeset iteration structure. */
 typedef struct {
 	list_t iters;             /*!< List of pending zone iterators. */
+	zone_tree_t *trees[4];    /*!< Poiters to zone trees to iterate over. */
+	size_t n_trees;           /*!< Their count. */
+	zone_tree_it_t it;        /*!< Zone tree iterator. */
 	const zone_node_t *node;  /*!< Current zone node. */
 	uint16_t node_pos;        /*!< Position in node. */
 } changeset_iter_t;
@@ -140,23 +142,22 @@ int changeset_remove_removal(changeset_t *ch, const knot_rrset_t *rrset);
 int changeset_merge(changeset_t *ch1, const changeset_t *ch2, int flags);
 
 /*!
- * \brief Remove from changeset those rdata which won't be added/removed from zone.
+ * \brief Get serial "from" of the changeset.
  *
- * \param zone  The zone the changeset is going to be applied on.
- * \param ch    The cheangeset to be fixed.
+ * \param ch   Changeset in question.
  *
- * \return KNOT_E*
+ * \return Its serial "from", or 0 if none.
  */
-int changeset_preapply_fix(const zone_contents_t *zone, changeset_t *ch);
+uint32_t changeset_from(const changeset_t *ch);
 
 /*!
- * \brief Remove from changeset records which are removed and added the same.
+ * \brief Get serial "to" of the changeset.
  *
- * \param ch  Changeset to be fixed.
+ * \param ch   Changeset in question.
  *
- * \return KNOT_E*
+ * \return Its serial "to", or 0 if none.
  */
-int changeset_cancelout(changeset_t *ch);
+uint32_t changeset_to(const changeset_t *ch);
 
 /*!
  * \brief Check the changes and SOA, ignoring possibly updated SOA serial.
@@ -171,32 +172,6 @@ int changeset_cancelout(changeset_t *ch);
  */
 bool changeset_differs_just_serial(const changeset_t *ch);
 
-/*!
- * \brief Loads zone contents from botstrap changeset.
- *
- * \param ch  Changeset to load from, will be freed!
- * \param out Zone contents.
- *
- * \return KNOT_E*
- */
-int changeset_to_contents(changeset_t *ch, zone_contents_t **out);
-
-/*!
- * \brief Creates a bootstrap changeset from zone.
- *
- * \param contents  Contents to include, will be freed!
- *
- * \return Changeset, which shall be freed with changeset_from_contents_free()
- */
-changeset_t *changeset_from_contents(const zone_contents_t *contents);
-
-/*!
- * \brief Frees single changeset.
- *
- * \param ch  Changeset from changeset_from_contents() to free.
- */
-void changeset_from_contents_free(changeset_t *ch);
-
 /*!
  * \brief Clears changesets in list. Changesets are not free'd. Legacy.
  *
