@@ -16,6 +16,7 @@
 
 #include "knot/catalog/interpret.h"
 #include "knot/common/log.h"
+#include "knot/common/systemd.h"
 #include "knot/dnssec/zone-events.h"
 #include "knot/updates/zone-update.h"
 #include "knot/zone/adds_tree.h"
@@ -921,17 +922,21 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 		bool incr_valid = update->flags & UPDATE_INCREMENTAL;
 		const char *msg_valid = incr_valid ? "incremental " : "";
 
-		ret = knot_dnssec_validate_zone(update, conf, incr_valid);
+		ret = knot_dnssec_validate_zone(update, conf, 0, incr_valid);
 		if (ret != KNOT_EOK) {
 			log_zone_error(update->zone->name, "DNSSEC, %svalidation failed (%s)",
 			               msg_valid, knot_strerror(ret));
-			char name_str[KNOT_DNAME_TXT_MAXLEN], type_str[16];
+			char type_str[16];
+			knot_dname_txt_storage_t name_str;
 			if (knot_dname_to_str(name_str, update->validation_hint.node, sizeof(name_str)) != NULL &&
 			    knot_rrtype_to_string(update->validation_hint.rrtype, type_str, sizeof(type_str)) >= 0) {
 				log_zone_error(update->zone->name, "DNSSEC, validation hint: %s %s",
 				               name_str, type_str);
 			}
 			discard_adds_tree(update);
+			if (conf->cache.srv_dbus_event & DBUS_EVENT_ZONE_INVALID) {
+				systemd_emit_zone_invalid(update->zone->name);
+			}
 			return ret;
 		} else {
 			log_zone_info(update->zone->name, "DNSSEC, %svalidation successful", msg_valid);
@@ -992,6 +997,11 @@ int zone_update_commit(conf_t *conf, zone_update_t *update)
 		zone_events_schedule_now(update->zone, ZONE_EVENT_FLUSH);
 	}
 
+	if (conf->cache.srv_dbus_event & DBUS_EVENT_ZONE_UPDATED) {
+		systemd_emit_zone_updated(update->zone->name,
+		                          zone_contents_serial(update->zone->contents));
+	}
+
 	memset(update, 0, sizeof(*update));
 
 	return KNOT_EOK;
