@@ -35,62 +35,40 @@ serial_cmp_result_t serial_compare(uint32_t s1, uint32_t s2)
 	return diffbrief2result[diffbrief];
 }
 
-static uint32_t serial_next_date(uint32_t current)
+static uint32_t serial_dateserial(uint32_t current)
 {
-	uint32_t next = current + 1;
-
 	struct tm now;
 	time_t current_time = time(NULL);
 	struct tm *gmtime_result = gmtime_r(&current_time, &now);
 	if (gmtime_result == NULL) {
-		return next;
+		return current;
 	}
-
-	uint32_t yyyyMMdd00 = (1900 + now.tm_year) * 1000000 +
-	                      (   1 + now.tm_mon ) *   10000 +
-	                      (       now.tm_mday) *     100;
-
-	if (next < yyyyMMdd00) {
-		next = yyyyMMdd00;
-	}
-
-	return next;
+	return (1900 + now.tm_year) * 1000000 +
+	       (   1 + now.tm_mon ) *   10000 +
+	       (       now.tm_mday) *     100;
 }
 
-uint32_t serial_next(uint32_t current, int policy)
+uint32_t serial_next(uint32_t current, int policy, uint32_t must_increment)
 {
-	uint32_t candidate;
+	uint32_t minimum;
 	switch (policy) {
 	case SERIAL_POLICY_INCREMENT:
-		return current + 1;
+		minimum = current;
+		break;
 	case SERIAL_POLICY_UNIXTIME:
-		candidate = time(NULL);
-		if (serial_compare(candidate, current) != SERIAL_GREATER) {
-			return current + 1;
-		} else {
-			return candidate;
-		}
+		minimum = time(NULL);
+		break;
 	case SERIAL_POLICY_DATESERIAL:
-		return serial_next_date(current);
+		minimum = serial_dateserial(current);
+		break;
 	default:
 		assert(0);
 		return 0;
 	}
-}
-
-bool serial_must_increment(uint32_t current, int policy)
-{
-	switch (policy) {
-	case SERIAL_POLICY_INCREMENT:
-		return false;
-	case SERIAL_POLICY_UNIXTIME:
-		return true;
-	case SERIAL_POLICY_DATESERIAL:
-		// Trim off the serial parts from YYYYMMDDnn.
-		return (current / 100) < (serial_next_date(current) / 100);
-	default:
-		assert(0);
-		return false;
+	if (serial_compare(minimum, current) != SERIAL_GREATER) {
+		return current + must_increment;
+	} else {
+		return minimum;
 	}
 }
 
