@@ -58,6 +58,7 @@ enum timer_id {
 	TIMER_NEXT_DS_PUSH,
 	TIMER_CATALOG_MEMBER,
 	TIMER_LAST_NOTIFIED,
+	TIMER_LAST_REFR_OK,
 };
 
 #define TIMER_SIZE (sizeof(uint8_t) + sizeof(uint64_t))
@@ -85,6 +86,7 @@ static int deserialize_timers(zone_timers_t *timers_ptr,
 		case TIMER_LAST_FLUSH:     timers.last_flush = value; break;
 		case TIMER_LAST_REFRESH:   timers.last_refresh = value; break;
 		case TIMER_NEXT_REFRESH:   timers.next_refresh = value; break;
+		case TIMER_LAST_REFR_OK:   timers.last_refresh_ok = value; break;
 		case TIMER_LAST_NOTIFIED:  timers.last_notified_serial = value; break;
 		case TIMER_LAST_RESALT:    timers.last_resalt = value; break;
 		case TIMER_NEXT_DS_CHECK:  timers.next_ds_check = value; break;
@@ -108,11 +110,12 @@ static void txn_write_timers(knot_lmdb_txn_t *txn, const knot_dname_t *zone,
                              const zone_timers_t *timers)
 {
 	MDB_val k = { knot_dname_size(zone), (void *)zone };
-	MDB_val v = knot_lmdb_make_key("BLBLBLBLBLBLBLBLBL",
+	MDB_val v = knot_lmdb_make_key("BLBLBLBLBLBLBLBLBLBL",
 		TIMER_SOA_EXPIRE,    (uint64_t)timers->soa_expire,
 		TIMER_LAST_FLUSH,    (uint64_t)timers->last_flush,
 		TIMER_LAST_REFRESH,  (uint64_t)timers->last_refresh,
 		TIMER_NEXT_REFRESH,  (uint64_t)timers->next_refresh,
+		TIMER_LAST_REFR_OK,  (uint64_t)timers->last_refresh_ok,
 		TIMER_LAST_NOTIFIED, timers->last_notified_serial,
 		TIMER_LAST_RESALT,   (uint64_t)timers->last_resalt,
 		TIMER_NEXT_DS_CHECK, (uint64_t)timers->next_ds_check,
