@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,7 +21,6 @@
 #include "libknot/consts.h"
 #include "libknot/errcode.h"
 #include "libknot/packet/wire.h"
-#include "contrib/macros.h"
 
 typedef struct {
 	zone_tree_apply_cb_t func;
@@ -53,7 +52,7 @@ zone_tree_t *zone_tree_create(bool use_binodes)
 	return t;
 }
 
-zone_tree_t *zone_tree_dup(zone_tree_t *from)
+zone_tree_t *zone_tree_cow(zone_tree_t *from)
 {
 	zone_tree_t *to = calloc(1, sizeof(*to));
 	if (to == NULL) {
@@ -70,6 +69,26 @@ zone_tree_t *zone_tree_dup(zone_tree_t *from)
 	return to;
 }
 
+static trie_val_t nocopy(const trie_val_t val, _unused_ knot_mm_t *mm)
+{
+	return val;
+}
+
+zone_tree_t *zone_tree_shallow_copy(zone_tree_t *from)
+{
+	zone_tree_t *to = calloc(1, sizeof(*to));
+	if (to == NULL) {
+		return to;
+	}
+	to->flags = from->flags;
+	to->trie = trie_dup(from->trie, nocopy, NULL);
+	if (to->trie == NULL) {
+		free(to);
+		to = NULL;
+	}
+	return to;
+}
+
 int zone_tree_insert(zone_tree_t *tree, zone_node_t **node)
 {
 	if (tree == NULL || node == NULL || *node == NULL) {
