@@ -367,14 +367,14 @@ void zone_clear_preferred_master(zone_t *zone)
 	pthread_mutex_unlock(&zone->preferred_lock);
 }
 
-void zone_set_flag(zone_t *zone, zone_flag_t flag)
+static void set_flag(zone_t *zone, zone_flag_t flag, bool remove)
 {
 	if (zone == NULL) {
 		return;
 	}
 
 	pthread_mutex_lock(&zone->preferred_lock); // this mutex seems OK to be reused for this
-	zone->flags |= flag;
+	zone->flags = remove ? (zone->flags & ~flag) : (zone->flags | flag);
 	pthread_mutex_unlock(&zone->preferred_lock);
 
 	if (flag & ZONE_IS_CATALOG) {
@@ -382,6 +382,16 @@ void zone_set_flag(zone_t *zone, zone_flag_t flag)
 	}
 }
 
+void zone_set_flag(zone_t *zone, zone_flag_t flag)
+{
+	return set_flag(zone, flag, false);
+}
+
+void zone_unset_flag(zone_t *zone, zone_flag_t flag)
+{
+	return set_flag(zone, flag, true);
+}
+
 zone_flag_t zone_get_flag(zone_t *zone, zone_flag_t flag, bool clear)
 {
 	if (zone == NULL) {
@@ -497,10 +507,10 @@ static void log_try_addr_error(const zone_t *zone, const char *remote_name,
 {
 	char addr_str[SOCKADDR_STRLEN] = { 0 };
 	sockaddr_tostr(addr_str, sizeof(addr_str), remote_addr);
-	log_zone_debug(zone->name, "%s%s%s, address %s, failed (%s)", err_str,
-	               (remote_name != NULL ? ", remote " : ""),
-	               (remote_name != NULL ? remote_name : ""),
-	               addr_str, knot_strerror(ret));
+	log_zone_info(zone->name, "%s%s%s, address %s, failed (%s)", err_str,
+	              (remote_name != NULL ? ", remote " : ""),
+	              (remote_name != NULL ? remote_name : ""),
+	              addr_str, knot_strerror(ret));
 }
 
 int zone_master_try(conf_t *conf, zone_t *zone, zone_master_cb callback,
