@@ -78,7 +78,7 @@ static const struct error errors[] = {
 	{ KNOT_ENORECORD,    "no such record in zone found" },
 	{ KNOT_EISRECORD,    "such record already exists in zone" },
 	{ KNOT_ENOMASTER,    "no usable master" },
-	{ KNOT_EPREREQ,      "UPDATE prerequisity not met" },
+	{ KNOT_EPREREQ,      "UPDATE prerequisite not met" },
 	{ KNOT_ETTL,         "TTL mismatch" },
 	{ KNOT_ENOXFR,       "transfer was not sent" },
 	{ KNOT_EDENIED,      "not allowed" },
@@ -101,6 +101,7 @@ static const struct error errors[] = {
 	{ KNOT_ENOPARAM,     "missing parameter" },
 	{ KNOT_EXPARAM,      "parameter conflict" },
 	{ KNOT_EEMPTYZONE,   "zone is empty" },
+	{ KNOT_EUNREACH,     "remote known to be unreachable" },
 
 	{ KNOT_GENERAL_ERROR, "unknown general error" },
 
