@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -26,11 +26,13 @@
 #pragma once
 
 #include "libknot/version.h"
+#include "libknot/attribute.h"
 #include "libknot/cookies.h"
 #include "libknot/codes.h"
 #include "libknot/consts.h"
 #include "libknot/descriptor.h"
 #include "libknot/dname.h"
+#include "libknot/dynarray.h"
 #include "libknot/endian.h"
 #include "libknot/errcode.h"
 #include "libknot/error.h"
@@ -63,11 +65,7 @@
 #include "libknot/rrtype/rrsig.h"
 #include "libknot/rrtype/soa.h"
 #include "libknot/rrtype/tsig.h"
+#include "libknot/rrtype/zonemd.h"
 #include "libknot/wire.h"
-#if 0
-#include "libknot/xdp/xdp.h"
-#include "libknot/xdp/bpf-consts.h"
-#include "libknot/xdp/eth.h"
-#endif
 
 /*! @} */
