@@ -1,4 +1,4 @@
-/*  Copyright (C) 2017 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -11,32 +11,35 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
+    along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
+
 /*!
  * \file
  *
  * \brief Functions for manipulating the EDNS OPT pseudo-RR.
  *
- * \addtogroup libknot
+ * \addtogroup rrtype
  * @{
  */
 
 #pragma once
 
 #include <assert.h>
+#include <sys/socket.h>
 
 #include "libknot/consts.h"
 #include "libknot/rrset.h"
-
-/* Forward decls */
-typedef struct knot_pkt knot_pkt_t;
+#include "libknot/wire.h"
 
 /*! \brief Constants related to EDNS. */
-enum knot_edns_const {
+enum {
 	/*! \brief Supported EDNS version. */
 	KNOT_EDNS_VERSION = 0,
 
+	/*! \brief Bit mask for DO bit. */
+	KNOT_EDNS_DO_MASK = (uint32_t)(1 << 15),
+
 	/*! \brief Minimal UDP payload with EDNS enabled. */
 	KNOT_EDNS_MIN_UDP_PAYLOAD    = 512,
 	/*! \brief Minimal payload when using DNSSEC (RFC4035/sec.3). */
@@ -50,14 +53,27 @@ enum knot_edns_const {
 	KNOT_EDNS_EXT_RCODE_POS            = 5,
 	/*! \brief EDNS OPTION header size. */
 	KNOT_EDNS_OPTION_HDRLEN            = 4,
-	/*! \brief Maximal edns client subnet data size (IPv6). */
-	KNOT_EDNS_MAX_OPTION_CLIENT_SUBNET = 20,
+
 	/*! \brief Maximal size of EDNS client subnet address in bytes (IPv6). */
 	KNOT_EDNS_CLIENT_SUBNET_ADDRESS_MAXLEN = 16,
 
+	/*! \brief Default EDNS alignment size for a query. */
+	KNOT_EDNS_ALIGNMENT_QUERY_DEFALT     = 128,
+	/*! \brief Default EDNS alignment size for a response. */
+	KNOT_EDNS_ALIGNMENT_RESPONSE_DEFAULT = 468,
+
+	/*! \brief Current EDNS cookie version. */
+	KNOT_EDNS_COOKIE_VERSION       = 1,
+	/*! \brief EDNS client cookie size. */
+	KNOT_EDNS_COOKIE_CLNT_SIZE     = 8,
+	/*! \brief EDNS minimum server cookie size. */
+	KNOT_EDNS_COOKIE_SRVR_MIN_SIZE = 8,
+	/*! \brief EDNS maximum server cookie size. */
+	KNOT_EDNS_COOKIE_SRVR_MAX_SIZE = 32,
+
 	/*! \brief NSID option code. */
 	KNOT_EDNS_OPTION_NSID          = 3,
-	/*! \brief EDNS client subnet option code. */
+	/*! \brief EDNS Client subnet option code. */
 	KNOT_EDNS_OPTION_CLIENT_SUBNET = 8,
 	/*! \brief EDNS DNS Cookie option code. */
 	KNOT_EDNS_OPTION_COOKIE        = 10,
@@ -67,6 +83,48 @@ enum knot_edns_const {
 	KNOT_EDNS_OPTION_PADDING       = 12,
 	/*! \brief EDNS Chain query option code. */
 	KNOT_EDNS_OPTION_CHAIN         = 13,
+
+	/*! \brief EDNS Extended error code. */
+	KNOT_EDNS_OPTION_EDE           = 15,
+	/*! \brief The minimal length for EDE option including option header. */
+	KNOT_EDNS_EDE_MIN_LENGTH       = 6,
+
+	/*! \brief Maximal currently known option code. */
+	KNOT_EDNS_MAX_OPTION_CODE      = 17,
+};
+
+/*!
+ * \brief Extended error codes as in EDNS option #15.
+ * \warning Don't mix this up with EDNS extended errcode.
+ * \note The default -1 value must be filtered out before storing to uint16_t ;)
+ */
+enum {
+	KNOT_EDNS_EDE_NONE             = -1,
+	KNOT_EDNS_EDE_OTHER            = 0,
+	KNOT_EDNS_EDE_DNSKEY_ALG       = 1,
+	KNOT_EDNS_EDE_DS_DIGEST        = 2,
+	KNOT_EDNS_EDE_STALE            = 3,
+	KNOT_EDNS_EDE_FORGED           = 4,
+	KNOT_EDNS_EDE_INDETERMINATE    = 5,
+	KNOT_EDNS_EDE_BOGUS            = 6,
+	KNOT_EDNS_EDE_SIG_EXPIRED      = 7,
+	KNOT_EDNS_EDE_SIG_NOTYET       = 8,
+	KNOT_EDNS_EDE_DNSKEY_MISS      = 9,
+	KNOT_EDNS_EDE_RRSIG_MISS       = 10,
+	KNOT_EDNS_EDE_DNSKEY_BIT       = 11,
+	KNOT_EDNS_EDE_NSEC_MISS        = 12,
+	KNOT_EDNS_EDE_CACHED_ERR       = 13,
+	KNOT_EDNS_EDE_NOT_READY        = 14,
+	KNOT_EDNS_EDE_BLOCKED          = 15,
+	KNOT_EDNS_EDE_CENSORED         = 16,
+	KNOT_EDNS_EDE_FILTERED         = 17,
+	KNOT_EDNS_EDE_PROHIBITED       = 18,
+	KNOT_EDNS_EDE_STALE_NXD        = 19,
+	KNOT_EDNS_EDE_NOTAUTH          = 20,
+	KNOT_EDNS_EDE_NOTSUP           = 21,
+	KNOT_EDNS_EDE_NREACH_AUTH      = 22,
+	KNOT_EDNS_EDE_NETWORK          = 23,
+	KNOT_EDNS_EDE_INV_DATA         = 24,
 };
 
 /* Helpers for splitting extended RCODE. */
@@ -90,36 +148,52 @@ int knot_edns_init(knot_rrset_t *opt_rr, uint16_t max_pld,
 /*!
  * \brief Returns size of the OPT RR in wire format.
  *
+ * \warning This function does not check the parameter, so ensure to check it
+ *          before calling the function. It must not be NULL.
+ *
  * \param opt_rr  OPT RR to count the wire size of.
  *
  * \return Size of the OPT RR in bytes.
  */
-size_t knot_edns_wire_size(knot_rrset_t *opt_rr);
+static inline
+size_t knot_edns_wire_size(knot_rrset_t *opt_rr)
+{
+	assert(opt_rr != NULL);
+	return KNOT_EDNS_MIN_SIZE + opt_rr->rrs.rdata->len;
+}
 
 /*!
  * \brief Returns the Max UDP payload value stored in the OPT RR.
  *
  * \warning This function does not check the parameter, so ensure to check it
  *          before calling the function. It must not be NULL.
- * \note There is an assert() for debug checking of the parameter.
  *
  * \param opt_rr  OPT RR to get the value from.
  *
  * \return Max UDP payload in bytes.
  */
-uint16_t knot_edns_get_payload(const knot_rrset_t *opt_rr);
+static inline
+uint16_t knot_edns_get_payload(const knot_rrset_t *opt_rr)
+{
+	assert(opt_rr != NULL);
+	return opt_rr->rclass;
+}
 
 /*!
  * \brief Sets the Max UDP payload field in the OPT RR.
  *
  * \warning This function does not check the parameter, so ensure to check it
  *          before calling the function. It must not be NULL.
- * \note There is an assert() for debug checking of the parameter.
  *
  * \param opt_rr   OPT RR to set the value to.
  * \param payload  UDP payload in bytes.
  */
-void knot_edns_set_payload(knot_rrset_t *opt_rr, uint16_t payload);
+static inline
+void knot_edns_set_payload(knot_rrset_t *opt_rr, uint16_t payload)
+{
+	assert(opt_rr != NULL);
+	opt_rr->rclass = payload;
+}
 
 /*!
  * \brief Returns the Extended RCODE stored in the OPT RR.
@@ -147,7 +221,8 @@ uint8_t knot_edns_get_ext_rcode(const knot_rrset_t *opt_rr);
  *
  * \return 12-bit Extended RCODE.
  */
-static inline uint16_t knot_edns_whole_rcode(uint8_t ext_rcode, uint8_t rcode)
+static inline
+uint16_t knot_edns_whole_rcode(uint8_t ext_rcode, uint8_t rcode)
 {
 	uint16_t high = ext_rcode;
 	return (high << 4) | rcode;
@@ -171,9 +246,10 @@ void knot_edns_set_ext_rcode(knot_rrset_t *opt_rr, uint8_t ext_rcode);
  * \param opt_rr     Position of the OPT RR in packet.
  * \param ext_rcode  Higher 8 bits of Extended RCODE.
  */
-static inline void knot_edns_set_ext_rcode_wire(uint8_t *opt_rr,
-                                                uint8_t ext_rcode)
+static inline
+void knot_edns_set_ext_rcode_wire(uint8_t *opt_rr, uint8_t ext_rcode)
 {
+	assert(opt_rr != NULL);
 	*(opt_rr + KNOT_EDNS_EXT_RCODE_POS) = ext_rcode;
 }
 
@@ -207,52 +283,33 @@ void knot_edns_set_version(knot_rrset_t *opt_rr, uint8_t version);
  *
  * \warning This function does not check the parameter, so ensure to check it
  *          before calling the function. It must not be NULL.
- * \note There is an assert() for debug checking of the parameter.
  *
  * \param opt_rr  OPT RR to get the DO bit from.
  *
  * \return true if the DO bit is set.
  * \return false if the DO bit is not set.
  */
-bool knot_edns_do(const knot_rrset_t *opt_rr);
+static inline
+bool knot_edns_do(const knot_rrset_t *opt_rr)
+{
+	assert(opt_rr != NULL);
+	return opt_rr->ttl & KNOT_EDNS_DO_MASK;
+}
 
 /*!
  * \brief Sets the DO bit in the OPT RR.
  *
  * \warning This function does not check the parameter, so ensure to check it
  *          before calling the function. It must not be NULL.
- * \note There is an assert() for debug checking of the parameter.
  *
  * \param opt_rr  OPT RR to set the DO bit in.
  */
-void knot_edns_set_do(knot_rrset_t *opt_rr);
-
-/*!
- * \brief Removes all EDNS options with given \a code.
- *
- * \param[in] opt_rr  OPT RR structure to remove the options from.
- * \param[in] code    Option code.
- *
- * \return Error code, KNOT_EOK if successful (even if nothing removed).
- */
-int knot_edns_remove_options(knot_rrset_t *opt_rr, uint16_t code);
-
-/*!
- * \brief Adds EDNS option into the package with empty (zeroed) content.
- *
- * \note All other occurrences of the option type will be removed.
- *
- * \param[in]  opt_rr    OPT RR structure to reserve the option in.
- * \param[in]  code      Option code.
- * \param[in]  size      Desired option size.
- * \param[out] wire_ptr  Pointer to reserved option data (can be NULL).
- * \param[in]  mm        Memory context.
- *
- * \return Error code, KNOT_EOK if successful.
- */
-int knot_edns_reserve_unique_option(knot_rrset_t *opt_rr, uint16_t code,
-                                    uint16_t size, uint8_t **wire_ptr,
-                                    knot_mm_t *mm);
+static inline
+void knot_edns_set_do(knot_rrset_t *opt_rr)
+{
+	assert(opt_rr != NULL);
+	opt_rr->ttl |= KNOT_EDNS_DO_MASK;
+}
 
 /*!
  * \brief Add EDNS option into the package with empty (zeroed) content.
@@ -272,7 +329,6 @@ int knot_edns_reserve_option(knot_rrset_t *opt_rr, uint16_t code,
  * \brief Adds EDNS Option to the OPT RR.
  *
  * \note The function now supports adding empty OPTION (just having its code).
- *       This does not make much sense now with NSID, but may be ok use later.
  *
  * \param opt_rr  OPT RR structure to add the Option to.
  * \param code    Option code.
@@ -286,49 +342,66 @@ int knot_edns_reserve_option(knot_rrset_t *opt_rr, uint16_t code,
 int knot_edns_add_option(knot_rrset_t *opt_rr, uint16_t code,
                          uint16_t size, const uint8_t *data, knot_mm_t *mm);
 
-/*!
- * \brief Checks if the OPT RR contains Option with the specified code.
- *
- * \param opt_rr OPT RR structure to check for the Option in.
- * \param code Option code to check for.
- *
- * \retval true if the OPT RR contains Option with Option code \a code.
- * \retval false otherwise.
- */
-bool knot_edns_has_option(const knot_rrset_t *opt_rr, uint16_t code);
-
 /*!
  * \brief Searches the OPT RR for option with the specified code.
  *
- * \param opt_rr  OPT RR structure to search for the Option in.
- * \param code    Option code to search for.
+ * \param opt_rr    OPT RR structure to search in.
+ * \param code      Option code to search for.
+ * \param previous  (Optional) Previously returned option to start searching from,
+ *                  to be able to return another option with the same code.
  *
  * \retval pointer to option if found
  * \retval NULL otherwise.
  */
-uint8_t *knot_edns_get_option(const knot_rrset_t *opt_rr, uint16_t code);
+uint8_t *knot_edns_get_option(const knot_rrset_t *opt_rr, uint16_t code,
+                              const uint8_t *previous);
+
+/*!
+ * \brief Pointers to every option in the OPT RR wire.
+ */
+typedef struct {
+	uint8_t *ptr[KNOT_EDNS_MAX_OPTION_CODE + 1];
+} knot_edns_options_t;
+
+/*!
+ * \brief Initializes pointers to options in a given OPT RR.
+ *
+ * \note If the OPT RR has no options, the output is NULL.
+ *
+ * \param opt_rr  OPT RR structure to be used.
+ * \param out     Structure to be initialized.
+ * \param mm      Memory context.
+ *
+ * \return Error code, KNOT_EOK if successful.
+ */
+int knot_edns_get_options(knot_rrset_t *opt_rr, knot_edns_options_t **out,
+                          knot_mm_t *mm);
 
 /*!
  * \brief Returns the option code.
  *
- * \warning No safety checks are performed on the supplied data.
- *
  * \param opt  EDNS option (including code, length and data portion).
  *
- * \retval EDNS option code
+ * \return EDNS option code
  */
-uint16_t knot_edns_opt_get_code(const uint8_t *opt);
+static inline uint16_t knot_edns_opt_get_code(const uint8_t *opt)
+{
+	assert(opt != NULL);
+	return knot_wire_read_u16(opt);
+}
 
 /*!
  * \brief Returns the option data length.
  *
- * \warning No safety checks are performed on the supplied data.
- *
  * \param opt  EDNS option (including code, length and data portion).
  *
- * \retval EDNS option length
+ * \return EDNS option length
  */
-uint16_t knot_edns_opt_get_length(const uint8_t *opt);
+static inline uint16_t knot_edns_opt_get_length(const uint8_t *opt)
+{
+	assert(opt != NULL);
+	return knot_wire_read_u16(opt + sizeof(uint16_t));
+}
 
 /*!
  * \brief Returns pointer to option data.
@@ -344,28 +417,6 @@ static inline uint8_t *knot_edns_opt_get_data(uint8_t *opt)
 	return opt + KNOT_EDNS_OPTION_HDRLEN;
 }
 
-/*!
- * \brief Checks OPT RR semantics.
- *
- * Checks whether RDATA are OK, i.e. that all OPTIONs have proper lengths.
- *
- * \param opt_rr  OPT RR to check.
- *
- * \return true if passed, false if failed
- */
-bool knot_edns_check_record(knot_rrset_t *opt_rr);
-
-/*!
- * \brief Computes a reasonable Padding data length for a given packet and opt RR.
- *
- * \param pkt     DNS Packet prepared and otherwise ready to go, no OPT yet added.
- * \param opt_rr  OPT RR, not yet including padding.
- *
- * \return Required padding length or -1 if padding not required.
- */
-int knot_edns_default_padding_size(const knot_pkt_t *pkt,
-                                   const knot_rrset_t *opt_rr);
-
 /*!
  * \brief Computes additional Padding data length for required packet alignment.
  *
@@ -375,29 +426,16 @@ int knot_edns_default_padding_size(const knot_pkt_t *pkt,
  *
  * \return Required padding length or -1 if padding not required.
  */
-static inline int knot_edns_alignment_size(size_t current_pkt_size,
-                                           size_t current_opt_size,
-                                           size_t block_size)
-{
-	assert(current_opt_size > 0);
-	assert(block_size > 0);
-
-	size_t current_size = current_pkt_size + current_opt_size;
-	if (current_size % block_size == 0) {
-		return -1;
-	}
-
-	size_t modulo = (current_size + KNOT_EDNS_OPTION_HDRLEN) % block_size;
-
-	return (modulo == 0) ? 0 : block_size - modulo;
-}
+int knot_edns_alignment_size(size_t current_pkt_size,
+                             size_t current_opt_size,
+                             size_t block_size);
 
 /*!
  * \brief EDNS Client Subnet content.
  *
  * \see draft-ietf-dnsop-edns-client-subnet
  */
-struct knot_edns_client_subnet {
+typedef struct {
 	/*! \brief FAMILY */
 	uint16_t family;
 	/*! \brief SOURCE PREFIX-LENGTH */
@@ -406,10 +444,7 @@ struct knot_edns_client_subnet {
 	uint8_t scope_len;
 	/*! \brief ADDRESS */
 	uint8_t address[KNOT_EDNS_CLIENT_SUBNET_ADDRESS_MAXLEN];
-};
-
-typedef struct knot_edns_client_subnet knot_edns_client_subnet_t;
-struct sockaddr_storage;
+} knot_edns_client_subnet_t;
 
 /*!
  * \brief Get the wire size of the EDNS Client Subnet option.
@@ -418,7 +453,7 @@ struct sockaddr_storage;
  *
  * \return Size of the EDNS option data.
  */
-size_t knot_edns_client_subnet_size(const knot_edns_client_subnet_t *ecs);
+uint16_t knot_edns_client_subnet_size(const knot_edns_client_subnet_t *ecs);
 
 /*!
  * \brief Write EDNS Client Subnet data from the ECS structure to wire.
@@ -429,7 +464,7 @@ size_t knot_edns_client_subnet_size(const knot_edns_client_subnet_t *ecs);
  *
  * \return Error code, KNOT_EOK if successful.
  */
-int knot_edns_client_subnet_write(uint8_t *option, size_t option_len,
+int knot_edns_client_subnet_write(uint8_t *option, uint16_t option_len,
                                   const knot_edns_client_subnet_t *ecs);
 
 /*!
@@ -476,7 +511,7 @@ int knot_edns_client_subnet_get_addr(struct sockaddr_storage *addr,
  *
  * \return Size of the EDNS option data.
  */
-size_t knot_edns_keepalive_size(uint16_t timeout);
+uint16_t knot_edns_keepalive_size(uint16_t timeout);
 
 /*!
  * \brief Writes EDNS TCP Keepalive wire data.
@@ -487,7 +522,7 @@ size_t knot_edns_keepalive_size(uint16_t timeout);
  *
  * \return Error code, KNOT_EOK if successful.
  */
-int knot_edns_keepalive_write(uint8_t *option, size_t option_len, uint16_t timeout);
+int knot_edns_keepalive_write(uint8_t *option, uint16_t option_len, uint16_t timeout);
 
 /*!
  * \brief Parses EDNS TCP Keepalive wire data.
@@ -508,7 +543,7 @@ int knot_edns_keepalive_parse(uint16_t *timeout, const uint8_t *option,
  *
  * \return Size of the EDNS option data or 0 if invalid input.
  */
-size_t knot_edns_chain_size(const knot_dname_t *point);
+uint16_t knot_edns_chain_size(const knot_dname_t *point);
 
 /*!
  * \brief Writes EDNS Chain wire data.
@@ -519,7 +554,7 @@ size_t knot_edns_chain_size(const knot_dname_t *point);
  *
  * \return Error code, KNOT_EOK if successful.
  */
-int knot_edns_chain_write(uint8_t *option, size_t option_len,
+int knot_edns_chain_write(uint8_t *option, uint16_t option_len,
                           const knot_dname_t *point);
 
 /*!
@@ -528,10 +563,64 @@ int knot_edns_chain_write(uint8_t *option, size_t option_len,
  * \param[out] point       EDNS Chain closest trusted point.
  * \param[in]  option      EDNS option data.
  * \param[in]  option_len  EDNS option size.
+ * \param[in]  mm          Memory context.
  *
  * \return Error code, KNOT_EOK if successful.
  */
 int knot_edns_chain_parse(knot_dname_t **point, const uint8_t *option,
-                          uint16_t option_len);
+                          uint16_t option_len, knot_mm_t *mm);
+
+/*!
+ * \brief DNS Cookie content.
+ */
+typedef struct {
+	uint8_t data[KNOT_EDNS_COOKIE_SRVR_MAX_SIZE]; /*!< Cookie data. */
+	uint16_t len; /*!< Cookie length. */
+} knot_edns_cookie_t;
+
+/*!
+ * \brief Get size of the EDNS Cookie option wire size.
+ *
+ * \param[in] cc  Client cookie.
+ * \param[in] sc  Server cookie (can be NULL).
+ *
+ * \return Size of the EDNS option data or 0 if invalid input.
+ */
+uint16_t knot_edns_cookie_size(const knot_edns_cookie_t *cc,
+                               const knot_edns_cookie_t *sc);
+
+/*!
+ * \brief Writes EDNS cookie wire data.
+ *
+ * \param[out] option      EDNS option data buffer.
+ * \param[in]  option_len  EDNS option data buffer size.
+ * \param[in]  cc          EDNS client cookie.
+ * \param[in]  sc          EDNS server cookie (can be NULL).
+ *
+ * \return Error code, KNOT_EOK if successful.
+ */
+int knot_edns_cookie_write(uint8_t *option, uint16_t option_len,
+                           const knot_edns_cookie_t *cc,
+                           const knot_edns_cookie_t *sc);
+
+/*!
+ * \brief Parses EDNS Cookie wire data.
+ *
+ * \param[out] cc          EDNS client cookie.
+ * \param[out] sc          EDNS server cookie.
+ * \param[in]  option      EDNS option data.
+ * \param[in]  option_len  EDNS option size.
+ *
+ * \return Error code, KNOT_EOK if successful.
+ */
+int knot_edns_cookie_parse(knot_edns_cookie_t *cc, knot_edns_cookie_t *sc,
+                           const uint8_t *option, uint16_t option_len);
+
+/*!
+ * \brief Returns description of option #15 extended error code.
+ *
+ * \warning Don't use with ext_errcode from knot_edns_get_ext_rcode().
+ */
+const char *knot_edns_ede_strerr(uint16_t exterr_code);
 
 /*! @} */
