@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -14,6 +14,15 @@
     along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
+/*!
+ * \file
+ *
+ * \brief XDP socket interface.
+ *
+ * \addtogroup xdp
+ * @{
+ */
+
 #pragma once
 
 #include <bpf/xsk.h>
@@ -66,8 +75,14 @@ struct knot_xdp_socket {
 	/*! Interface context. */
 	const struct kxsk_iface *iface;
 
+	/*! If non-NULL, it's a mocked socket with this send function. */
+	int (*send_mock)(struct knot_xdp_socket *, const knot_xdp_msg_t[], uint32_t, uint32_t *);
+
 	/*! The kernel has to be woken up by a syscall indication. */
 	bool kernel_needs_wakeup;
+
+	/*! The limit of frame size. */
+	unsigned frame_limit;
 };
 
 /*!
@@ -110,3 +125,5 @@ int kxsk_socket_start(const struct kxsk_iface *iface, uint32_t listen_port,
  * \param iface  Interface context.
  */
 void kxsk_socket_stop(const struct kxsk_iface *iface);
+
+/*! @} */
