@@ -1,4 +1,4 @@
-/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -23,17 +23,19 @@
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
+#include <time.h>
 #include <unistd.h>
 
 #include "libknot/attribute.h"
 #include "libknot/endian.h"
 #include "libknot/errcode.h"
 #include "libknot/xdp/bpf-user.h"
+#include "libknot/xdp/eth.h"
+#include "libknot/xdp/msg_init.h"
+#include "libknot/xdp/protocols.h"
 #include "libknot/xdp/xdp.h"
 #include "contrib/macros.h"
-
-/* Don't fragment flag. */
-#define	IP_DF 0x4000
+#include "contrib/net.h"
 
 #define FRAME_SIZE 2048
 #define UMEM_FRAME_COUNT_RX 4096
@@ -42,6 +44,9 @@
 #define UMEM_RING_LEN_TX (UMEM_FRAME_COUNT_TX * 2)
 #define UMEM_FRAME_COUNT (UMEM_FRAME_COUNT_RX + UMEM_FRAME_COUNT_TX)
 
+#define ALLOC_RETRY_NUM   15
+#define ALLOC_RETRY_DELAY 20 // In nanoseconds.
+
 /* With recent compilers we statically check #defines for settings that
  * get refused by AF_XDP drivers (in current versions, at least). */
 #if (__STDC_VERSION__ >= 201112L)
@@ -56,48 +61,10 @@ _Static_assert((FRAME_SIZE == 4096 || FRAME_SIZE == 2048)
 	, "Incorrect #define combination for AF_XDP.");
 #endif
 
-/*! \brief The memory layout of IPv4 umem frame. */
-struct udpv4 {
-	union {
-		uint8_t bytes[1];
-		struct {
-			struct ethhdr eth; // No VLAN support; CRC at the "end" of .data!
-			struct iphdr ipv4;
-			struct udphdr udp;
-			uint8_t data[];
-		} __attribute__((packed));
-	};
-};
-
-/*! \brief The memory layout of IPv6 umem frame. */
-struct udpv6 {
-	union {
-		uint8_t bytes[1];
-		struct {
-			struct ethhdr eth; // No VLAN support; CRC at the "end" of .data!
-			struct ipv6hdr ipv6;
-			struct udphdr udp;
-			uint8_t data[];
-		} __attribute__((packed));
-	};
-};
-
-/*! \brief The memory layout of each umem frame. */
 struct umem_frame {
-	union {
-		uint8_t bytes[FRAME_SIZE];
-		union {
-			struct udpv4 udpv4;
-			struct udpv6 udpv6;
-		};
-	};
+	uint8_t bytes[FRAME_SIZE];
 };
 
-_public_
-const size_t KNOT_XDP_PAYLOAD_OFFSET4 = offsetof(struct udpv4, data) + offsetof(struct umem_frame, udpv4);
-_public_
-const size_t KNOT_XDP_PAYLOAD_OFFSET6 = offsetof(struct udpv6, data) + offsetof(struct umem_frame, udpv6);
-
 static int configure_xsk_umem(struct kxsk_umem **out_umem)
 {
 	/* Allocate memory and call driver to create the UMEM. */
@@ -218,6 +185,12 @@ int knot_xdp_init(knot_xdp_socket_t **socket, const char *if_name, int if_queue,
 		return ret;
 	}
 
+	(*socket)->frame_limit = FRAME_SIZE;
+	ret = knot_eth_mtu(if_name);
+	if (ret > 0) {
+		(*socket)->frame_limit = MIN((unsigned)ret, (*socket)->frame_limit);
+	}
+
 	ret = kxsk_socket_start(iface, listen_port, (*socket)->xsk);
 	if (ret != KNOT_EOK) {
 		xsk_socket__delete((*socket)->xsk);
@@ -237,6 +210,10 @@ void knot_xdp_deinit(knot_xdp_socket_t *socket)
 	if (socket == NULL) {
 		return;
 	}
+	if (unlikely(socket->send_mock != NULL)) {
+		free(socket);
+		return;
+	}
 
 	kxsk_socket_stop(socket->iface);
 	xsk_socket__delete(socket->xsk);
@@ -267,7 +244,7 @@ static void tx_free_relative(struct kxsk_umem *umem, uint64_t addr_relative)
 _public_
 void knot_xdp_send_prepare(knot_xdp_socket_t *socket)
 {
-	if (socket == NULL) {
+	if (socket == NULL || unlikely(socket->send_mock != NULL)) {
 		return;
 	}
 
@@ -289,201 +266,81 @@ void knot_xdp_send_prepare(knot_xdp_socket_t *socket)
 	xsk_ring_cons__release(cq, completed);
 }
 
-static struct umem_frame *alloc_tx_frame(struct kxsk_umem *umem)
+static struct umem_frame *alloc_tx_frame(knot_xdp_socket_t *socket)
 {
-	if (unlikely(umem->tx_free_count == 0)) {
-		return NULL;
+	if (unlikely(socket->send_mock != NULL)) {
+		return malloc(sizeof(struct umem_frame));
+	}
+
+	const struct timespec delay = { .tv_nsec = ALLOC_RETRY_DELAY };
+	struct kxsk_umem *umem = socket->umem;
+
+	for (int i = 0; unlikely(umem->tx_free_count == 0); i++) {
+		if (i == ALLOC_RETRY_NUM) {
+			return NULL;
+		}
+		nanosleep(&delay, NULL);
+		knot_xdp_send_prepare(socket);
 	}
 
 	uint32_t index = umem->tx_free_indices[--umem->tx_free_count];
 	return umem->frames + index;
 }
 
+static void prepare_payload(knot_xdp_msg_t *msg, void *uframe)
+{
+	size_t hdr_len = prot_write_hdrs_len(msg);
+	msg->payload.iov_base = uframe + hdr_len;
+	msg->payload.iov_len = FRAME_SIZE - hdr_len;
+}
+
 _public_
-int knot_xdp_send_alloc(knot_xdp_socket_t *socket, bool ipv6, knot_xdp_msg_t *out,
-                        const knot_xdp_msg_t *in_reply_to)
+int knot_xdp_send_alloc(knot_xdp_socket_t *socket, knot_xdp_msg_flag_t flags,
+                        knot_xdp_msg_t *out)
 {
 	if (socket == NULL || out == NULL) {
 		return KNOT_EINVAL;
 	}
 
-	size_t ofs = ipv6 ? KNOT_XDP_PAYLOAD_OFFSET6 : KNOT_XDP_PAYLOAD_OFFSET4;
-
-	struct umem_frame *uframe = alloc_tx_frame(socket->umem);
+	struct umem_frame *uframe = alloc_tx_frame(socket);
 	if (uframe == NULL) {
 		return KNOT_ENOMEM;
 	}
 
-	memset(out, 0, sizeof(*out));
-
-	out->payload.iov_base = ipv6 ? uframe->udpv6.data : uframe->udpv4.data;
-	out->payload.iov_len = MIN(UINT16_MAX, FRAME_SIZE - ofs);
-
-	const struct ethhdr *eth = (struct ethhdr *)uframe;
-	out->eth_from = (void *)&eth->h_source;
-	out->eth_to = (void *)&eth->h_dest;
-
-	if (in_reply_to != NULL) {
-		memcpy(out->eth_from, in_reply_to->eth_to, ETH_ALEN);
-		memcpy(out->eth_to, in_reply_to->eth_from, ETH_ALEN);
-
-		memcpy(&out->ip_from, &in_reply_to->ip_to, sizeof(out->ip_from));
-		memcpy(&out->ip_to, &in_reply_to->ip_from, sizeof(out->ip_to));
-	}
+	msg_init(out, flags);
+	prepare_payload(out, uframe);
 
 	return KNOT_EOK;
 }
 
-static uint16_t from32to16(uint32_t sum)
+_public_
+int knot_xdp_reply_alloc(knot_xdp_socket_t *socket, const knot_xdp_msg_t *query,
+                         knot_xdp_msg_t *out)
 {
-	sum = (sum & 0xffff) + (sum >> 16);
-	sum = (sum & 0xffff) + (sum >> 16);
-	return sum;
-}
-
-static uint16_t ipv4_checksum(const uint8_t *ipv4_hdr)
-{
-	const uint16_t *h = (const uint16_t *)ipv4_hdr;
-	uint32_t sum32 = 0;
-	for (int i = 0; i < 10; ++i) {
-		if (i != 5) {
-			sum32 += h[i];
-		}
+	if (socket == NULL || query == NULL || out == NULL) {
+		return KNOT_EINVAL;
 	}
-	return ~from32to16(sum32);
-}
 
-/* Checksum endianness implementation notes for ipv4_checksum() and udp_checksum_step().
- *
- * The basis for checksum is addition on big-endian 16-bit words, with bit 16 carrying
- * over to bit 0.  That can be viewed as first byte carrying to the second and the
- * second one carrying back to the first one, i.e. a symmetrical situation.
- * Therefore the result is the same even when arithmetics is done on litte-endian (!)
- */
-
-static void udp_checksum_step(size_t *result, const void *_data, size_t _data_len)
-{
-	assert(!(_data_len & 1));
-	const uint16_t *data = _data;
-	size_t len = _data_len / 2;
-	while (len-- > 0) {
-		*result += *data++;
+	struct umem_frame *uframe = alloc_tx_frame(socket);
+	if (uframe == NULL) {
+		return KNOT_ENOMEM;
 	}
+
+	msg_init_reply(out, query);
+	prepare_payload(out, uframe);
+
+	return KNOT_EOK;
 }
 
-static void udp_checksum_finish(size_t *result)
+static void free_unsent(knot_xdp_socket_t *socket, const knot_xdp_msg_t *msg)
 {
-	while (*result > 0xffff) {
-		*result = (*result & 0xffff) + (*result >> 16);
+	if (unlikely(socket->send_mock != NULL)) {
+		free(msg->payload.iov_base - prot_write_hdrs_len(msg));
+		return;
 	}
-	if (*result != 0xffff) {
-		*result = ~*result;
-	}
-}
-
-static uint8_t *msg_uframe_ptr(knot_xdp_socket_t *socket, const knot_xdp_msg_t *msg,
-                               /* Next parameters are just for debugging. */
-                               bool ipv6)
-{
-	uint8_t *uNULL = NULL;
-	uint8_t *uframe_p = uNULL + ((msg->payload.iov_base - NULL) & ~(FRAME_SIZE - 1));
-
-#ifndef NDEBUG
-	intptr_t pd = (uint8_t *)msg->payload.iov_base - uframe_p
-	              - (ipv6 ? KNOT_XDP_PAYLOAD_OFFSET6 : KNOT_XDP_PAYLOAD_OFFSET4);
-	/* This assertion might fire in some OK cases.  For example, the second branch
-	 * had to be added for cases with "emulated" AF_XDP support. */
-	assert(pd == XDP_PACKET_HEADROOM || pd == 0);
-
-	const uint8_t *umem_mem_start = socket->umem->frames->bytes;
-	const uint8_t *umem_mem_end = umem_mem_start + FRAME_SIZE * UMEM_FRAME_COUNT;
-	assert(umem_mem_start <= uframe_p && uframe_p < umem_mem_end);
-#endif
-	return uframe_p;
-}
-
-static void xsk_sendmsg_ipv4(knot_xdp_socket_t *socket, const knot_xdp_msg_t *msg,
-                             uint32_t index)
-{
-	uint8_t *uframe_p = msg_uframe_ptr(socket, msg, false);
-	struct umem_frame *uframe = (struct umem_frame *)uframe_p;
-	struct udpv4 *h = &uframe->udpv4;
-
-	const struct sockaddr_in *src_v4 = (const struct sockaddr_in *)&msg->ip_from;
-	const struct sockaddr_in *dst_v4 = (const struct sockaddr_in *)&msg->ip_to;
-	const uint16_t udp_len = sizeof(h->udp) + msg->payload.iov_len;
-
-	h->eth.h_proto = __constant_htons(ETH_P_IP);
-
-	h->ipv4.version  = IPVERSION;
-	h->ipv4.ihl      = 5;
-	h->ipv4.tos      = 0;
-	h->ipv4.tot_len  = htobe16(5 * 4 + udp_len);
-	h->ipv4.id       = 0;
-	h->ipv4.frag_off = 0;
-	h->ipv4.ttl      = IPDEFTTL;
-	h->ipv4.protocol = IPPROTO_UDP;
-	memcpy(&h->ipv4.saddr, &src_v4->sin_addr, sizeof(src_v4->sin_addr));
-	memcpy(&h->ipv4.daddr, &dst_v4->sin_addr, sizeof(dst_v4->sin_addr));
-	h->ipv4.check    = ipv4_checksum(h->bytes + sizeof(struct ethhdr));
-
-	h->udp.len    = htobe16(udp_len);
-	h->udp.source = src_v4->sin_port;
-	h->udp.dest   = dst_v4->sin_port;
-	h->udp.check  = 0; // Optional for IPv4 - not computed.
-
-	*xsk_ring_prod__tx_desc(&socket->tx, index) = (struct xdp_desc){
-		.addr = h->bytes - socket->umem->frames->bytes,
-		.len = KNOT_XDP_PAYLOAD_OFFSET4 + msg->payload.iov_len
-	};
-}
-
-static void xsk_sendmsg_ipv6(knot_xdp_socket_t *socket, const knot_xdp_msg_t *msg,
-                             uint32_t index)
-{
-	uint8_t *uframe_p = msg_uframe_ptr(socket, msg, true);
-	struct umem_frame *uframe = (struct umem_frame *)uframe_p;
-	struct udpv6 *h = &uframe->udpv6;
-
-	const struct sockaddr_in6 *src_v6 = (const struct sockaddr_in6 *)&msg->ip_from;
-	const struct sockaddr_in6 *dst_v6 = (const struct sockaddr_in6 *)&msg->ip_to;
-	const uint16_t udp_len = sizeof(h->udp) + msg->payload.iov_len;
-
-	h->eth.h_proto = __constant_htons(ETH_P_IPV6);
-
-	h->ipv6.version     = 6;
-	h->ipv6.priority    = 0;
-	memset(h->ipv6.flow_lbl, 0, sizeof(h->ipv6.flow_lbl));
-	h->ipv6.payload_len = htobe16(udp_len);
-	h->ipv6.nexthdr     = IPPROTO_UDP;
-	h->ipv6.hop_limit   = IPDEFTTL;
-	memcpy(&h->ipv6.saddr, &src_v6->sin6_addr, sizeof(src_v6->sin6_addr));
-	memcpy(&h->ipv6.daddr, &dst_v6->sin6_addr, sizeof(dst_v6->sin6_addr));
-
-	h->udp.len    = htobe16(udp_len);
-	h->udp.source = src_v6->sin6_port;
-	h->udp.dest   = dst_v6->sin6_port;
-	h->udp.check  = 0; // Mandatory for IPv6 - computed afterwards.
-
-	size_t chk = 0;
-	udp_checksum_step(&chk, &h->ipv6.saddr, sizeof(h->ipv6.saddr));
-	udp_checksum_step(&chk, &h->ipv6.daddr, sizeof(h->ipv6.daddr));
-	udp_checksum_step(&chk, &h->udp.len, sizeof(h->udp.len));
-	__be16 version = htobe16(h->ipv6.nexthdr);
-	udp_checksum_step(&chk, &version, sizeof(version));
-	udp_checksum_step(&chk, &h->udp, sizeof(h->udp));
-	size_t padded_len = msg->payload.iov_len;
-	if (padded_len & 1) {
-		((uint8_t *)msg->payload.iov_base)[padded_len++] = 0;
-	}
-	udp_checksum_step(&chk, msg->payload.iov_base, padded_len);
-	udp_checksum_finish(&chk);
-	h->udp.check = chk;
-
-	*xsk_ring_prod__tx_desc(&socket->tx, index) = (struct xdp_desc){
-		.addr = h->bytes - socket->umem->frames->bytes,
-		.len = KNOT_XDP_PAYLOAD_OFFSET6 + msg->payload.iov_len
-	};
+	uint64_t addr_relative = (uint8_t *)msg->payload.iov_base
+	                         - socket->umem->frames->bytes;
+	tx_free_relative(socket->umem, addr_relative);
 }
 
 _public_
@@ -493,6 +350,13 @@ int knot_xdp_send(knot_xdp_socket_t *socket, const knot_xdp_msg_t msgs[],
 	if (socket == NULL || msgs == NULL || sent == NULL) {
 		return KNOT_EINVAL;
 	}
+	if (unlikely(socket->send_mock != NULL)) {
+		int ret = socket->send_mock(socket, msgs, count, sent);
+		for (uint32_t i = 0; i < count; ++i) {
+			free_unsent(socket, &msgs[i]);
+		}
+		return ret;
+	}
 
 	/* Now we want to do something close to
 	 *   xsk_ring_prod__reserve(&socket->tx, count, *idx)
@@ -507,15 +371,19 @@ int knot_xdp_send(knot_xdp_socket_t *socket, const knot_xdp_msg_t msgs[],
 	for (uint32_t i = 0; i < count; ++i) {
 		const knot_xdp_msg_t *msg = &msgs[i];
 
-		if (msg->payload.iov_len && msg->ip_from.sin6_family == AF_INET) {
-			xsk_sendmsg_ipv4(socket, msg, idx++);
-		} else if (msg->payload.iov_len && msg->ip_from.sin6_family == AF_INET6) {
-			xsk_sendmsg_ipv6(socket, msg, idx++);
+		if (empty_msg(msg)) {
+			free_unsent(socket, msg);
 		} else {
-			/* Some problem; we just ignore this message. */
-			uint64_t addr_relative = (uint8_t *)msg->payload.iov_base
-			                         - socket->umem->frames->bytes;
-			tx_free_relative(socket->umem, addr_relative);
+			size_t hdr_len = prot_write_hdrs_len(msg);
+			size_t tot_len = hdr_len + msg->payload.iov_len;
+			uint8_t *msg_beg = msg->payload.iov_base - hdr_len;
+			uint16_t mss = MIN(socket->frame_limit - hdr_len, KNOT_TCP_MSS);
+			prot_write_eth(msg_beg, msg, msg_beg + tot_len, mss);
+
+			*xsk_ring_prod__tx_desc(&socket->tx, idx++) = (struct xdp_desc) {
+				.addr = msg_beg - socket->umem->frames->bytes,
+				.len = tot_len,
+			};
 		}
 	}
 
@@ -528,6 +396,15 @@ int knot_xdp_send(knot_xdp_socket_t *socket, const knot_xdp_msg_t msgs[],
 	return KNOT_EOK;
 }
 
+_public_
+void knot_xdp_send_free(knot_xdp_socket_t *socket, const knot_xdp_msg_t msgs[],
+                        uint32_t count)
+{
+	for (uint32_t i = 0; i < count; i++) {
+		free_unsent(socket, &msgs[i]);
+	}
+}
+
 _public_
 int knot_xdp_send_finish(knot_xdp_socket_t *socket)
 {
@@ -565,78 +442,9 @@ int knot_xdp_send_finish(knot_xdp_socket_t *socket)
 	 */
 }
 
-static void rx_desc(knot_xdp_socket_t *socket, const struct xdp_desc *desc,
-                    knot_xdp_msg_t *msg)
-{
-	uint8_t *uframe_p = socket->umem->frames->bytes + desc->addr;
-	const struct ethhdr *eth = (struct ethhdr *)uframe_p;
-	const struct iphdr *ip4 = NULL;
-	const struct ipv6hdr *ip6 = NULL;
-	const struct udphdr *udp = NULL;
-
-	switch (eth->h_proto) {
-	case __constant_htons(ETH_P_IP):
-		ip4 = (struct iphdr *)(uframe_p + sizeof(struct ethhdr));
-		// Next conditions are ensured by the BPF filter.
-		assert(ip4->version == 4);
-		assert(ip4->frag_off == 0 ||
-		       ip4->frag_off == __constant_htons(IP_DF));
-		assert(ip4->protocol == IPPROTO_UDP);
-		// IPv4 header checksum is not verified!
-		udp = (struct udphdr *)(uframe_p + sizeof(struct ethhdr) +
-		                        ip4->ihl * 4);
-		break;
-	case __constant_htons(ETH_P_IPV6):
-		ip6 = (struct ipv6hdr *)(uframe_p + sizeof(struct ethhdr));
-		// Next conditions are ensured by the BPF filter.
-		assert(ip6->version == 6);
-		assert(ip6->nexthdr == IPPROTO_UDP);
-		udp = (struct udphdr *)(uframe_p + sizeof(struct ethhdr) +
-		                        sizeof(struct ipv6hdr));
-		break;
-	default:
-		assert(0);
-		msg->payload.iov_len = 0;
-		return;
-	}
-	// UDP checksum is not verified!
-
-	assert(eth && (!!ip4 != !!ip6) && udp);
-
-	// Process the packet; ownership is passed on, beware of holding frames.
-
-	msg->payload.iov_base = (uint8_t *)udp + sizeof(struct udphdr);
-	msg->payload.iov_len = be16toh(udp->len) - sizeof(struct udphdr);
-
-	msg->eth_from = (void *)&eth->h_source;
-	msg->eth_to = (void *)&eth->h_dest;
-
-	if (ip4 != NULL) {
-		struct sockaddr_in *src_v4 = (struct sockaddr_in *)&msg->ip_from;
-		struct sockaddr_in *dst_v4 = (struct sockaddr_in *)&msg->ip_to;
-		memcpy(&src_v4->sin_addr, &ip4->saddr, sizeof(src_v4->sin_addr));
-		memcpy(&dst_v4->sin_addr, &ip4->daddr, sizeof(dst_v4->sin_addr));
-		src_v4->sin_port = udp->source;
-		dst_v4->sin_port = udp->dest;
-		src_v4->sin_family = AF_INET;
-		dst_v4->sin_family = AF_INET;
-	} else {
-		assert(ip6);
-		struct sockaddr_in6 *src_v6 = (struct sockaddr_in6 *)&msg->ip_from;
-		struct sockaddr_in6 *dst_v6 = (struct sockaddr_in6 *)&msg->ip_to;
-		memcpy(&src_v6->sin6_addr, &ip6->saddr, sizeof(src_v6->sin6_addr));
-		memcpy(&dst_v6->sin6_addr, &ip6->daddr, sizeof(dst_v6->sin6_addr));
-		src_v6->sin6_port = udp->source;
-		dst_v6->sin6_port = udp->dest;
-		src_v6->sin6_family = AF_INET6;
-		dst_v6->sin6_family = AF_INET6;
-		// Flow label is ignored.
-	}
-}
-
 _public_
 int knot_xdp_recv(knot_xdp_socket_t *socket, knot_xdp_msg_t msgs[],
-                  uint32_t max_count, uint32_t *count)
+                  uint32_t max_count, uint32_t *count, size_t *wire_size)
 {
 	if (socket == NULL || msgs == NULL || count == NULL) {
 		return KNOT_EINVAL;
@@ -651,7 +459,19 @@ int knot_xdp_recv(knot_xdp_socket_t *socket, knot_xdp_msg_t msgs[],
 	assert(available <= max_count);
 
 	for (uint32_t i = 0; i < available; ++i) {
-		rx_desc(socket, xsk_ring_cons__rx_desc(&socket->rx, idx++), &msgs[i]);
+		knot_xdp_msg_t *msg = &msgs[i];
+		const struct xdp_desc *desc = xsk_ring_cons__rx_desc(&socket->rx, idx++);
+		uint8_t *uframe_p = socket->umem->frames->bytes + desc->addr;
+
+		void *payl_end, *payl_start = prot_read_eth(uframe_p, msg, &payl_end);
+
+		msg->payload.iov_base = payl_start;
+		msg->payload.iov_len = payl_end - payl_start;
+		msg->mss = MIN(msg->mss, FRAME_SIZE - (payl_start - (void *)uframe_p));
+
+		if (wire_size != NULL) {
+			(*wire_size) += desc->len;
+		}
 	}
 
 	xsk_ring_cons__release(&socket->rx, available);
@@ -660,6 +480,11 @@ int knot_xdp_recv(knot_xdp_socket_t *socket, knot_xdp_msg_t msgs[],
 	return KNOT_EOK;
 }
 
+static uint8_t *msg_uframe_ptr(const knot_xdp_msg_t *msg)
+{
+	return NULL + ((msg->payload.iov_base - NULL) & ~(FRAME_SIZE - 1));
+}
+
 _public_
 void knot_xdp_recv_finish(knot_xdp_socket_t *socket, const knot_xdp_msg_t msgs[],
                           uint32_t count)
@@ -676,8 +501,7 @@ void knot_xdp_recv_finish(knot_xdp_socket_t *socket, const knot_xdp_msg_t msgs[]
 	assert(reserved == count);
 
 	for (uint32_t i = 0; i < reserved; ++i) {
-		uint8_t *uframe_p = msg_uframe_ptr(socket, &msgs[i],
-		                                   msgs[i].ip_from.sin6_family == AF_INET6);
+		uint8_t *uframe_p = msg_uframe_ptr(&msgs[i]);
 		uint64_t offset = uframe_p - umem->frames->bytes;
 		*xsk_ring_prod__fill_addr(fq, idx++) = offset;
 	}
