@@ -250,7 +250,9 @@ static void print_ede(const uint8_t *data, uint16_t len)
 		printf("(malformed)");
 		return;
 	}
-	uint16_t errcode = be16toh(*(uint16_t *)data);
+	uint16_t errcode;
+	memcpy(&errcode, data, sizeof(errcode));
+	errcode = be16toh(errcode);
 	const char *strerr = knot_edns_ede_strerr(errcode);
 	if (len > 2) {
 		printf("%hu (%s): '%.*s'", errcode, strerr, (int)(len - 2), data + 2);
