@@ -42,13 +42,16 @@ static void print_help(void)
 	       "  %s -l\n"
 	       "\n"
 	       "Parameters:\n"
+	       "  -b, --brief              List keys briefly.\n"
 	       "  -c, --config <file>      Use a textual configuration file.\n"
 	       "                            (default %s)\n"
 	       "  -C, --confdb <dir>       Use a binary configuration database directory.\n"
 	       "                            (default %s)\n"
-	       "  -d, --dir <path>         Use specified KASP database path and default configuration.\n"
+	       "  -D, --dir <path>         Use specified KASP database path and default configuration.\n"
 	       "  -t, --tsig <name> [alg]  Generate a TSIG key.\n"
 	       "  -l, --list               List all zones that have at least one key in KASP database.\n"
+	       "  -x, --mono               Don't color the output.\n"
+	       "  -X, --color              Force output colorization in the --brief mode.\n"
 	       "  -h, --help               Print the program help.\n"
 	       "  -V, --version            Print the program version.\n"
 	       "\n"
@@ -114,7 +117,8 @@ static void print_help(void)
 	       PROGRAM_NAME, PROGRAM_NAME, PROGRAM_NAME, PROGRAM_NAME, CONF_DEFAULT_FILE, CONF_DEFAULT_DBDIR);
 }
 
-static int key_command(int argc, char *argv[], int opt_ind, knot_lmdb_db_t *kaspdb)
+static int key_command(int argc, char *argv[], int opt_ind, knot_lmdb_db_t *kaspdb,
+                       keymgr_list_params_t *list_params)
 {
 	if (argc < opt_ind + 2) {
 		ERROR("zone name and/or command not specified\n");
@@ -197,13 +201,13 @@ static int key_command(int argc, char *argv[], int opt_ind, knot_lmdb_db_t *kasp
 			}
 		}
 	} else if (strcmp(argv[1], "list") == 0) {
-		knot_time_print_t format = TIME_PRINT_UNIX;
+		list_params->format = TIME_PRINT_UNIX;
 		if (argc > 2 && strcmp(argv[2], "human") == 0) {
-			format = TIME_PRINT_HUMAN_MIXED;
+			list_params->format = TIME_PRINT_HUMAN_MIXED;
 		} else if (argc > 2 && strcmp(argv[2], "iso") == 0) {
-			format = TIME_PRINT_ISO8601;
+			list_params->format = TIME_PRINT_ISO8601;
 		}
-		ret = keymgr_list_keys(&kctx, format);
+		ret = keymgr_list_keys(&kctx, list_params);
 		print_ok_on_succes = false;
 	} else if (strcmp(argv[1], "ds") == 0 || strcmp(argv[1], "dnskey") == 0) {
 		int (*generate_rr)(const knot_dname_t *, const knot_kasp_key_t *) = keymgr_generate_dnskey;
@@ -249,6 +253,7 @@ static int key_command(int argc, char *argv[], int opt_ind, knot_lmdb_db_t *kasp
 	} else if (strcmp(argv[1], "show-offline") == 0) {
 		CHECK_MISSING_ARG("Timestamp from not specified");
 		ret = keymgr_print_offline_records(&kctx, argv[2], argc > 3 ? argv[3] : NULL);
+		print_ok_on_succes = false;
 	} else if (strcmp(argv[1], "del-offline") == 0) {
 		CHECK_MISSING_ARG2("Timestamps from-to not specified");
 		ret = keymgr_delete_offline_records(&kctx, argv[2], argv[3]);
@@ -361,14 +366,15 @@ static bool conf_initialized = false; // This is a singleton as well as conf() i
 
 int main(int argc, char *argv[])
 {
-	int ret, just_list = 0;
-
 	struct option opts[] = {
 		{ "config",  required_argument, NULL, 'c' },
 		{ "confdb",  required_argument, NULL, 'C' },
-		{ "dir",     required_argument, NULL, 'd' },
+		{ "dir",     required_argument, NULL, 'D' },
 		{ "tsig",    required_argument, NULL, 't' },
+		{ "brief",   no_argument,       NULL, 'b' },
 		{ "list",    no_argument,       NULL, 'l' },
+		{ "mono",    no_argument,       NULL, 'x' },
+		{ "color",   no_argument,       NULL, 'X' },
 		{ "help",    no_argument,       NULL, 'h' },
 		{ "version", no_argument,       NULL, 'V' },
 		{ NULL }
@@ -376,8 +382,14 @@ int main(int argc, char *argv[])
 
 	tzset();
 
+	int ret;
+	bool just_list = false;
+	keymgr_list_params_t list_params = { 0 };
+
+	list_params.color = isatty(STDOUT_FILENO);
+
 	int opt = 0, parm = 0;
-	while ((opt = getopt_long(argc, argv, "hVd:c:C:t:l", opts, NULL)) != -1) {
+	while ((opt = getopt_long(argc, argv, "hVd:D:c:C:t:lbxX", opts, NULL)) != -1) {
 		switch (opt) {
 		case 'h':
 			print_help();
@@ -386,6 +398,9 @@ int main(int argc, char *argv[])
 			print_version(PROGRAM_NAME);
 			return EXIT_SUCCESS;
 		case 'd':
+			WARNING("keymgr -d is deprecated, use -D option instead\n");
+			// FALLTHROUGH
+		case 'D':
 			CHECK_CONF_UNINIT
 			if (!init_conf(NULL) || !init_conf_blank(optarg)) {
 				return EXIT_FAILURE;
@@ -413,7 +428,16 @@ int main(int argc, char *argv[])
 			}
 			return (ret == KNOT_EOK ? EXIT_SUCCESS : EXIT_FAILURE);
 		case 'l':
-			just_list = 1;
+			just_list = true;
+			break;
+		case 'b':
+			list_params.brief = true;
+			break;
+		case 'x':
+			list_params.color = false;
+			break;
+		case 'X':
+			list_params.color = true;
 			break;
 		default:
 			print_help();
@@ -429,7 +453,7 @@ int main(int argc, char *argv[])
 			   init_conf(NULL) && init_confile(CONF_DEFAULT_FILE)) {
 			// initialized conf from default confile
 		} else {
-			ERROR("couldn't initialize configuration, please provide -c, -C, or -d option\n");
+			ERROR("couldn't initialize configuration, please provide -c, -C, or -D option\n");
 			return EXIT_FAILURE;
 		}
 	}
@@ -445,7 +469,7 @@ int main(int argc, char *argv[])
 	if (just_list) {
 		ret = keymgr_list_zones(&kaspdb);
 	} else {
-		ret = key_command(argc, argv, optind, &kaspdb);
+		ret = key_command(argc, argv, optind, &kaspdb, &list_params);
 	}
 
 	knot_lmdb_deinit(&kaspdb);
