@@ -1,4 +1,4 @@
-/*  Copyright (C) 2021 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2022 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include "knot/conf/conf.h"
 #include "knot/dnssec/zone-events.h"
 #include "knot/updates/zone-update.h"
+#include "knot/zone/adjust.h"
 #include "knot/zone/zone-load.h"
 #include "knot/zone/zonefile.h"
 #include "utils/common/params.h"
@@ -56,6 +57,7 @@ static void print_help(void)
 	       "Parameters:\n"
 	       " -o, --outdir <dir_name>  Output directory.\n"
 	       " -r, --rollover           Allow key rollovers and NSEC3 re-salt.\n"
+	       " -v, --verify             Only verify if zone is signed correctly.\n"
 	       " -t, --time <timestamp>   Current time specification.\n"
 	       "                            (default current UNIX time)\n"
 	       " -h, --help               Print the program help.\n"
@@ -75,11 +77,13 @@ int main(int argc, char *argv[])
 	zone_sign_roll_flags_t rollover = 0;
 	int64_t timestamp = 0;
 	zone_sign_reschedule_t next_sign = { 0 };
+	bool check_only = false;
 
 	struct option opts[] = {
 		{ "config",    required_argument, NULL, 'c' },
 		{ "outdir",    required_argument, NULL, 'o' },
 		{ "rollover",  no_argument,       NULL, 'r' },
+		{ "verify" ,   no_argument,       NULL, 'v' },
 		{ "time",      required_argument, NULL, 't' },
 		{ "help",      no_argument,       NULL, 'h' },
 		{ "version",   no_argument,       NULL, 'V' },
@@ -89,7 +93,7 @@ int main(int argc, char *argv[])
 	tzset();
 
 	int opt;
-	while ((opt = getopt_long(argc, argv, "c:o:rt:hV", opts, NULL)) != -1) {
+	while ((opt = getopt_long(argc, argv, "c:o:rvt:hV", opts, NULL)) != -1) {
 		switch (opt) {
 		case 'c':
 			confile = optarg;
@@ -100,6 +104,9 @@ int main(int argc, char *argv[])
 		case 'r':
 			rollover = KEY_ROLL_ALLOW_ALL;
 			break;
+		case 'v':
+			check_only = true;
+			break;
 		case 't':
 			timestamp = atol(optarg);
 			if (timestamp <= 0) {
@@ -174,6 +181,31 @@ int main(int argc, char *argv[])
 		goto fail;
 	}
 
+	if (check_only) {
+		val = conf_zone_get(conf(), C_ADJUST_THR, zone_name);
+		ret = zone_adjust_full(up.new_cont, conf_int(&val));
+		if (ret != KNOT_EOK) {
+			printf("Failed to adjust the zone (%s)\n", knot_strerror(ret));
+			zone_update_clear(&up);
+			goto fail;
+		}
+
+		ret = knot_dnssec_validate_zone(&up, conf(), timestamp, false);
+		if (ret != KNOT_EOK) {
+			printf("DNSSEC validation failed (%s)\n", knot_strerror(ret));
+			char type_str[16];
+			knot_dname_txt_storage_t name_str;
+			if (knot_dname_to_str(name_str, up.validation_hint.node, sizeof(name_str)) != NULL &&
+			    knot_rrtype_to_string(up.validation_hint.rrtype, type_str, sizeof(type_str)) >= 0) {
+				printf("Affected node: '%s' type '%s'\n", name_str, type_str);
+			}
+		} else {
+			printf("DNSSEC validation successful\n");
+		}
+		zone_update_clear(&up);
+		goto fail;
+	}
+
 	kasp_db_ensure_init(&kasp_db, conf());
 	zone_struct->kaspdb = &kasp_db;
 
