@@ -1,4 +1,4 @@
-/*  Copyright (C) 2018 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
+/*  Copyright (C) 2020 CZ.NIC, z.s.p.o. <knot-dns@labs.nic.cz>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -96,7 +96,7 @@ static void check_key(const key_parameters_t *key_data, const dnssec_binary_t *d
 	ok(r == DNSSEC_EOK, "create signing context");
 	r = dnssec_sign_add(ctx, data);
 	ok(r == DNSSEC_EOK, "add data to be signed");
-	r = dnssec_sign_verify(ctx, signature);
+	r = dnssec_sign_verify(ctx, false, signature);
 	ok(r == DNSSEC_EOK, "signature verified");
 
 	// create new signature and self-validate
@@ -110,11 +110,11 @@ static void check_key(const key_parameters_t *key_data, const dnssec_binary_t *d
 		r = dnssec_sign_add(ctx, data);
 		ok(r == DNSSEC_EOK, "add data to be signed");
 		dnssec_binary_t new_signature = { 0 };
-		r = dnssec_sign_write(ctx, &new_signature);
+		r = dnssec_sign_write(ctx, DNSSEC_SIGN_NORMAL, &new_signature);
 		ok(r == DNSSEC_EOK, "write the signature");
 		ok(dnssec_binary_cmp(signature, &new_signature) == 0,
 		   "signature exact match");
-		r = dnssec_sign_verify(ctx, &new_signature);
+		r = dnssec_sign_verify(ctx, false, &new_signature);
 		ok(r == DNSSEC_EOK, "reverify the new signature");
 		dnssec_binary_free(&new_signature);
 	}
@@ -142,7 +142,7 @@ static void check_key(const key_parameters_t *key_data, const dnssec_binary_t *d
 	ok(r == DNSSEC_EOK, "add data (3)");
 
 	dnssec_binary_t new_signature = { 0 };
-	r = dnssec_sign_write(ctx, &new_signature);
+	r = dnssec_sign_write(ctx, DNSSEC_SIGN_NORMAL, &new_signature);
 	ok(r == DNSSEC_EOK, "write signature");
 
 	r = dnssec_sign_init(ctx);
@@ -152,7 +152,7 @@ static void check_key(const key_parameters_t *key_data, const dnssec_binary_t *d
 	r = dnssec_sign_add(ctx, &tmp);
 	ok(r == DNSSEC_EOK, "add data (4)");
 
-	r = dnssec_sign_verify(ctx, &new_signature);
+	r = dnssec_sign_verify(ctx, false, &new_signature);
 	ok(r == DNSSEC_EOK, "verify signature");
 
 	dnssec_binary_free(&new_signature);
